# Microsoft Access Snapshot Viewer
# CVE-2008-2463

import logging

log = logging.getLogger("Thug")


def PrintSnapshot(self, SnapshotPath = '', CompressedPath = ''):
    if SnapshotPath:
        self.SnapshotPath = SnapshotPath

    if CompressedPath:
        self.CompressedPath = CompressedPath

    msg = '[Microsoft Access Snapshot Viewer ActiveX] SnapshotPath : %s, CompressedPath: %s' % (self.SnapshotPath,
                                                                                                self.CompressedPath, )
    log.ThugLogging.add_behavior_warn(msg, 'CVE-2008-2463')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft Access Snapshot Viewer ActiveX",
                                      "Print Snapshot",
                                      forward = False,
                                      cve = 'CVE-2008-2463',
                                      data = {
                                                "SnapshotPath"  : self.SnapshotPath,
                                                "CompressedPath": self.CompressedPath
                                             }
                                     )
    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2008-2463", None)
    url = self.SnapshotPath

    try:
        self._window._navigator.fetch(url, redirect_type = "CVE-2008-2463")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[Microsoft Access Snapshot Viewer ActiveX] Fetch failed')



#!/usr/bin/env python
#
# HTTPSession.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import sys
import socket
import requests
import ssl

try:
    import urllib.parse as urlparse
    from urllib.parse import quote
except ImportError:
    import urlparse
    from urllib import quote

import logging
log = logging.getLogger("Thug")


class HTTPSession(object):
    def __init__(self, proxy = None):
        if proxy is None:
            proxy = log.ThugOpts.proxy

        self.__init_session(proxy)
        self.filecount = 0

    def __check_proxy_alive(self, hostname, port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((hostname, port))
        s.close()

    def __do_init_proxy(self, proxy):
        url = urlparse.urlparse(proxy)
        if not url.scheme:
            return False

        if not url.scheme.lower().startswith(('http', 'socks4', 'socks5')):
            return False

        try:
            self.__check_proxy_alive(url.hostname, url.port)
        except:  # pylint:disable=bare-except
            log.critical("[CRITICAL] Proxy not available. Aborting the analysis!")

            if log.ThugOpts.raise_for_proxy:
                raise ValueError("[CRITICAL] Proxy not available")

            sys.exit(0)

        self.session.proxies = {
            'http'  : proxy,
            'https' : proxy
        }

        return True

    def __init_proxy(self, proxy):
        if proxy is None:
            return

        if self.__do_init_proxy(proxy):
            return

        log.critical("[CRITICAL] Wrong proxy specified. Aborting the analysis!")
        sys.exit(0)

    def __init_session(self, proxy):
        self.session = requests.Session()
        self.__init_proxy(proxy)

    def _normalize_protocol_relative_url(self, window, url):
        if not url.startswith('//'):
            return url

        if window.url in ('about:blank', ):
            return 'http:%s' % (url, )

        base_url = urlparse.urlparse(window.url)
        if not base_url.scheme:
            return 'http:%s' % (url, )

        return "%s:%s" % (base_url.scheme, url)

    def _is_compatible(self, url, scheme):
        return url.startswith("%s:/" % (scheme, )) and not url.startswith("%s://" % (scheme, ))

    def _check_compatibility(self, url):
        for scheme in ("http", "https", ):
            if self._is_compatible(url, scheme):
                return "%s://%s" % (scheme, url.split("%s:/" % (scheme, ))[1], )

        return url

    def normalize_url(self, window, url):
        # Do not normalize Data URI scheme
        if url.lower().startswith('url='):
            return url

        # Check the URL is not broken (i.e. http:/www.google.com) and
        # fix it if the broken URL option is enabled.
        if log.ThugOpts.broken_url:
            url = self._check_compatibility(url)

        url = self._normalize_protocol_relative_url(window, url)
        url = quote(url, safe = "%/:=&?~#+!$,;'@()*[]")

        _url = urlparse.urlparse(url)

        # Check if a scheme handler is registered and calls the proper
        # handler in such case. This is how a real browser would handle
        # a specific scheme so if you want to add your own handler for
        # analyzing specific schemes the proper way to go is to define
        # a method named handle_<scheme> in the SchemeHandler and put
        # the logic within such method.
        handler = getattr(log.SchemeHandler, 'handle_%s' % (_url.scheme, ), None)
        if handler:
            handler(window, url)
            return None

        if not _url.netloc:
            _url = urlparse.urljoin(window.url, url)
            log.warning("[Navigator URL Translation] %s --> %s", url, _url)
            return _url

        return url

    def build_http_headers(self, window, personality, headers):
        http_headers = {
            'Cache-Control'   : 'no-cache',
            'Accept-Language' : 'en-US',
            'Accept'          : '*/*',
            'User-Agent'      :  personality
        }

        if window and window.url not in ('about:blank', ):
            http_headers['Referer'] = self.normalize_url(window, window.url)

        # REVIEW ME!
        # if window and window.doc.cookie:
        #    http_headers['Cookie'] = window.doc.cookie

        for name, value in headers.items():
            http_headers[name] = value

        return http_headers

    def fetch_ssl_certificate(self, url):
        if not log.ThugOpts.cert_logging:
            return

        _url = urlparse.urlparse(url)
        if _url.scheme not in ('https', ):
            return

        port = _url.port if _url.port else 443
        certificate = ssl.get_server_certificate((_url.netloc, port), ssl_version = ssl.PROTOCOL_SSLv23)
        log.ThugLogging.log_certificate(url, certificate)

    def fetch(self, url, method = "GET", window = None, personality = None, headers = None, body = None):
        if log.URLClassifier.filter(url):
            return None

        fetcher = getattr(self.session, method.lower(), None)
        if fetcher is None:
            log.ThugLogging.log_warning("Not supported method: %s" % (method, ))
            return None

        if headers is None:
            headers = dict()

        _headers = self.build_http_headers(window, personality, headers)

        try:
            response = fetcher(url,
                               headers = _headers,
                               timeout = log.ThugOpts.connect_timeout,
                               data    = body,
                               verify  = False)
        except requests.ConnectionError as e:
            log.ThugLogging.log_warning("[HTTPSession] {0}".format(e.message))

        self.filecount += 1

        if log.ThugOpts.web_tracking:
            log.WebTracking.inspect_response(response)

        return response

    def threshold_expired(self, url):
        if not log.ThugOpts.threshold:
            return False

        if self.filecount >= log.ThugOpts.threshold:
            log.ThugLogging.log_location(url, None, flags = {"error" : "Threshold Exceeded"})
            return True

        return False

    @property
    def no_fetch(self):
        return log.ThugOpts.no_fetch

    def about_blank(self, url):
        return url.lower() in ('about:blank', )


import unittest


class HTTPSessionTest(unittest.TestCase):
    def setUp(self):
        self.check_ip_url = "http://ifconfig.me/ip"

    def testHTTPSession(self):
        s = HTTPSession()
        r = s.session.get("http://www.google.com")
        self.assertTrue(r.ok)

    def testHTTPSessionSOCKS(self):
        stor = HTTPSession()
        rtor = stor.session.get("https://www.dan.me.uk/torlist/")
        tor_exit_nodes = rtor.text.split("\n")

        s = HTTPSession(proxy = "socks5://127.0.0.1:9050")
        r = s.session.get(self.check_ip_url)
        ipaddress = r.text.replace("\n", "")
        self.assertIn(ipaddress, tor_exit_nodes)

    def testHTTPSessionNotSupportedMethod(self):
        s = HTTPSession()
        r = s.fetch("http://www.google.com", method = "NOTSUPPORTED")
        self.assertEqual(r, None)

    def testHTTPSessionGET(self):
        s = HTTPSession()
        r = s.fetch("http://www.google.com")
        self.assertTrue(r.ok)

if __name__ == '__main__':
    unittest.main()



# Yahoo! Music Jukebox 2.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def AddBitmap(self, arg0, arg1, arg2, arg3, arg4, arg5):
    if len(arg1) > 256:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Yahoo! Music Jukebox ActiveX",
                                          "Overflow in AddBitmap")


def AddButton(self, arg0, arg1):
    if len(arg0) > 256:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Yahoo! Music Jukebox ActiveX",
                                          "Overflow in AddButton")


def AddImage(self, arg0, arg1):
    if len(arg0) > 256:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Yahoo! Music Jukebox ActiveX",
                                          "Overflow in AddImage")



#!/usr/bin/env python

import logging

log = logging.getLogger("Thug")

import bs4 as BeautifulSoup

from .Node import Node
from .NodeList import NodeList
from .DocumentFragment import DocumentFragment
from .DocumentType import DocumentType
from .Element import Element
from .Comment import Comment
from .Text import Text
from .CDATASection import CDATASection
from .Attr import Attr
from .EntityReference import EntityReference
from .ProcessingInstruction import ProcessingInstruction
from .Events.DocumentEvent import DocumentEvent
from .Views.DocumentView import DocumentView


class Document(Node, DocumentEvent, DocumentView):
    def __init__(self, doc):
        Node.__init__(self, doc)
        DocumentEvent.__init__(self, doc)
        DocumentView.__init__(self, doc)
        self.__init_personality()

    def __init_personality(self):
        self.__init_characterSet()

        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_characterSet(self):
        self._character_set = ""
        for meta in self.doc.find_all("meta"):
            if 'charset' in meta.attrs:
                self._characterSet = meta.attrs['charset'].upper()

    def __init_personality_IE(self):
        self.defaultCharset = self._defaultCharset

        if log.ThugOpts.Personality.browserMajorVersion > 7:
            self.querySelectorAll = self._querySelectorAll
            self.querySelector    = self._querySelector

        if log.ThugOpts.Personality.browserMajorVersion > 8:
            self.getElementsByClassName = self._getElementsByClassName
            self.characterSet           = self._characterSet
            self.inputEncoding          = self._inputEncoding
        else:
            self.charset                = self._characterSet

        if log.ThugOpts.Personality.browserMajorVersion > 10:
            self.__proto__ = None

    def __init_personality_Firefox(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName
        self.characterSet           = self._characterSet
        self.inputEncoding          = self._inputEncoding

    def __init_personality_Chrome(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName
        self.characterSet           = self._characterSet
        self.inputEncoding          = self._inputEncoding

    def __init_personality_Safari(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName
        self.characterSet           = self._characterSet
        self.inputEncoding          = self._inputEncoding

    def __init_personality_Opera(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName
        self.characterSet           = self._characterSet

    def _querySelectorAll(self, selectors):
        try:
            s = self.doc.select(selectors)
        except:  # pylint:disable=bare-except
            return NodeList(self.doc, [])

        return NodeList(self.doc, s)

    def _querySelector(self, selectors):
        from .DOMImplementation import DOMImplementation

        try:
            s = self.doc.select(selectors)
        except:  # pylint:disable=bare-except
            return None

        if s and s[0]:
            return DOMImplementation.createHTMLElement(self, s[0])

        return None

    def __str__(self):
        return str(self.doc)

    def __unicode__(self):
        return unicode(self.doc)

    def __repr__(self):
        return "<Document at 0x%08X>" % id(self)

    @property
    def nodeType(self):
        return Node.DOCUMENT_NODE

    @property
    def nodeName(self):
        return "#document"

    @property
    def nodeValue(self):
        return None

    @property
    def childNodes(self):
        return NodeList(self.doc, self.doc.contents)

    @property
    def doctype(self):
        for tag in self.doc:
            if isinstance(tag, BeautifulSoup.Declaration) and tag.startswith("DOCTYPE"):
                return DocumentType(self.doc, tag)

        return None

    @property
    def implementation(self):
        return self

    @property
    def documentElement(self):
        return Element(self, self.doc.find('html'))

    onCreateElement = None

    def getCharacterSet(self):
        return self._character_set

    def setCharacterSet(self, value):
        self._character_set = value

    _characterSet = property(getCharacterSet, setCharacterSet)

    @property
    def _inputEncoding(self):
        for meta in self.doc.find_all("meta"):
            if 'charset' in meta.attrs:
                return meta.attrs['charset'].upper()

        return ""

    @property
    def _defaultCharset(self):
        return "Windows-1252"

    def createElement(self, tagname, tagvalue = None):
        from .DOMImplementation import DOMImplementation

        # Internet Explorer 8 and below also support the syntax
        # document.createElement('<P>')
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
            if tagname.startswith('<') and '>' in tagname:
                tagname = tagname[1:].split('>')[0]

        element = DOMImplementation.createHTMLElement(self, BeautifulSoup.Tag(parser = self.doc, name = tagname))
        if self.onCreateElement:
            self.onCreateElement(element)  # pylint:disable=not-callable

        return element

    def createDocumentFragment(self):
        return DocumentFragment(self)

    def createTextNode(self, data):
        return Text(self, BeautifulSoup.NavigableString(data))

    def createComment(self, data):
        return Comment(self, BeautifulSoup.Comment(data))

    def createCDATASection(self, data):
        return CDATASection(self, BeautifulSoup.CData(data))

    def createProcessingInstruction(self, target, data):
        return ProcessingInstruction(self, target, BeautifulSoup.ProcessingInstruction(data))

    def createAttribute(self, name):
        return Attr(self, None, name)

    def createEntityReference(self, name):
        return EntityReference(self, name)

    def getElementsByTagName(self, tagname):
        if log.ThugOpts.Personality.isIE() and tagname in ('*', ):
            s = [p for p in self.doc.find_all(text = False)]
            return NodeList(self.doc, s)

        return NodeList(self.doc, self.doc.find_all(tagname.lower()))

    def _getElementsByClassName(self, classname):
        return NodeList(self.doc, self.doc.find_all(class_ = classname))

    # Introduced in DOM Level 2
    def getElementById(self, elementId):
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 8:
            return self._getElementById_IE67(elementId)

        return self._getElementById(elementId)

    def _getElementById(self, elementId):
        from .DOMImplementation import DOMImplementation

        tag = self.doc.find(id = elementId)
        return DOMImplementation.createHTMLElement(self, tag) if tag else None

    # Internet Explorer 6 and 7 getElementById is broken and returns
    # elements with 'id' or 'name' attributes equal to elementId
    def _getElementById_IE67(self, elementId):
        from .DOMImplementation import DOMImplementation

        def _match_tag(tag, p):
            return p in tag.attrs and tag.attrs[p] == elementId

        def match_tag(tag, _id):
            if _match_tag(tag, _id):
                return True

            return False

        def filter_tags_id(tag):
            return tag.has_attr('id')

        def filter_tags_name(tag):
            return tag.has_attr('name')

        for tag in self.doc.find_all(filter_tags_id):
            if match_tag(tag, 'id'):
                return DOMImplementation.createHTMLElement(self, tag)

        for tag in self.doc.find_all(filter_tags_name):
            if match_tag(tag, 'name'):
                return DOMImplementation.createHTMLElement(self, tag)

        return None

    # Introduced in DOM Level 2
    def importNode(self, importedNode, deep):
        # TODO
        pass

    # Modified in DOM Level 2
    @property
    def ownerDocument(self):
        return None

    def execCommand(self, commandIdentifier, userInterface = False, value = None):
        pass



#!/usr/bin/env python
#
# Location.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging

from .DFT import DFT
from .W3C import w3c

from .JSClass import JSClass

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

log = logging.getLogger("Thug")


class Location(JSClass):
    def __init__(self, window):
        self._window = window

    def toString(self):
        return self._window.url

    def __repr__(self):
        return self._window.url

    @property
    def parts(self):
        return urlparse.urlparse(self._window.url)

    def get_href(self):
        return self._window.url

    def set_href(self, url):
        from .Window import Window

        referer = self._window.url
        if referer == url:
            log.warning("Detected redirection from %s to %s... skipping", referer, url)
            return

        for p in log.ThugOpts.Personality:
            if log.ThugOpts.Personality[p]['userAgent'] == self._window._navigator.userAgent:
                break

        url = log.HTTPSession.normalize_url(self._window, url)
        log.ThugLogging.log_href_redirect(referer, url)

        doc    = w3c.parseString('')
        window = Window(referer, doc, personality = p)  # pylint:disable=undefined-loop-variable
        window = window.open(url)
        if not window:
            return

        # self._window.url = url
        dft = DFT(window)
        dft.run()

    href = property(get_href, set_href)

    @property
    def protocol(self):
        return self.parts.scheme

    @property
    def host(self):
        return self.parts.netloc

    @property
    def hostname(self):
        return self.parts.hostname

    @property
    def port(self):
        return self.parts.port

    @property
    def pathname(self):
        return self.parts.path

    @property
    def search(self):
        return self.parts.query

    @property
    def hash(self):
        return self.parts.fragment

    def assign(self, url):
        """Loads a new HTML document."""
        self._window.open(url)

    def reload(self, force = False):
        """Reloads the current page."""
        self._window.open(self._window.url)

    def replace(self, url):
        """Replaces the current document by loading another document at the specified URL."""
        self.href = url



#!/usr/bin/env python
#
# TestURLClassifier.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import unittest
import yara
import json


class TestURLClassifier(unittest.TestCase):
    rule_file = "../rules/urlclassifier.yar"
    test_file = "testurlclassifier.json"

    def setUp(self):
        self.rules = yara.compile(self.rule_file)

    def test(self):
        with open(self.test_file) as fd:
            tests = json.load(fd)

        for t in tests['tests']:
            self.assertEqual(str(self.rules.match(data = t['url'])), t['output'])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# MIMEHandler.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


import os
import logging
log = logging.getLogger("Thug")

import base64
import hashlib
import zipfile
import rarfile
import tempfile

try:
    from cStringIO import StringIO
except ImportError:
    try:
        from StringIO import StringIO
    except ImportError:
        from io import StringIO

SSDEEP = True
try:
    import ssdeep
except ImportError:
    SSDEEP = False

import bs4 as BeautifulSoup

PEEPDF = True
try:
    from peepdf.PDFCore import PDFParser, vulnsDict
except ImportError:
    PEEPDF = False

from datetime import datetime
from lxml import etree

ANDROGUARD = True
try:
    from androguard.core import androconf
    from androguard.core.bytecodes import apk
    from androguard.core.bytecodes import dvm
    from androguard.core.analysis import analysis
except ImportError:
    log.warning("[WARNING] Androguard not found - APK analysis disabled")
    ANDROGUARD = False


class MIMEHandler(dict):
    """
        MIMEHandler class is meant to allow registering MIME handlers the
        same way a real browser would do.

        The default handling for almost all Content-Types is to not further
        processing the downloaded content and this can be done by returning
        True from the Content-Type handler. The method `passthrough' is the
        default handler associated to almost all Content-Types with the few
        exceptions defined in the method `register_empty_handlers'.

        Two ways actually exist for further processing a downloaded content.
        The first one is returning False from the the Content-Type handler.
        The second one is having a None Content-Type handler which turns to
        be quite useful when an unknown Content-Type is served. In such case
        the __missing__ method will return None (thus enabling further content
        processing) and log the unknown Content-Type for convenience.

        This design is quite flexible because i.e. you can decide to instantiate
        your own PDF analysis system really quickly by simply defining a new
        application/pdf Content-Type handler.
    """
    mimetypes = ("application/download",
                 "application/envoy",
                 "application/exe",
                 "application/fractals",
                 "application/futuresplash",
                 "application/hta",
                 "application/internet-property-stream",
                 "application/java-archive",
                 "application/javascript",
                 "application/mac-binhex40",
                 "application/msword",
                 "application/octet-stream",
                 "application/oda",
                 "application/olescript",
                 "application/pdf",
                 "application/pics-rules",
                 "application/pkcs10",
                 "application/pkix-crl",
                 "application/postscript",
                 "application/rar",
                 "application/rtf",
                 "application/set-payment-initiation",
                 "application/set-registration-initiation",
                 "application/vnd.android.package-archive",
                 "application/vnd.ms-excel",
                 "application/vnd.ms-outlook",
                 "application/vnd.ms-pkicertstore",
                 "application/vnd.ms-pkiseccat",
                 "application/vnd.ms-pkistl",
                 "application/vnd.ms-powerpoint",
                 "application/vnd.ms-project",
                 "application/vnd.ms-works",
                 "application/winhlp",
                 "application/x-bcpio",
                 "application/x-bzip2",
                 "application/x-cdf",
                 "application/x-chrome-extension",
                 "application/x-compress",
                 "application/x-compressed",
                 "application/x-cpio",
                 "application/x-csh",
                 "application/x-director",
                 "application/x-dosexec",
                 "application/x-dvi",
                 "application/x-gtar",
                 "application/x-gzip",
                 "application/x-hdf",
                 "application/x-internet-signup",
                 "application/x-iphone",
                 "application/x-java-jnlp-file",
                 "application/x-javascript",
                 "application/x-latex",
                 "application/x-msaccess",
                 "application/x-mscardfile",
                 "application/x-msclip",
                 "application/x-msdos-program",
                 "application/x-msdownload",
                 "application/x-msmediaview",
                 "application/x-msmetafile",
                 "application/x-msmoney",
                 "application/x-mspublisher",
                 "application/x-msschedule",
                 "application/x-msterminal",
                 "application/x-mswrite",
                 "application/x-netcdf",
                 "application/x-perfmon",
                 "application/x-pkcs12",
                 "application/x-pkcs7-certificates",
                 "application/x-pkcs7-certreqresp",
                 "application/x-pkcs7-mime",
                 "application/x-pkcs7-signature",
                 "application/x-rar-compressed",
                 "application/x-sh",
                 "application/x-shar",
                 "application/x-shockwave-flash",
                 "application/x-silverlight-2",
                 "application/x-stuffit",
                 "application/x-sv4cpio",
                 "application/x-sv4crc",
                 "application/x-tar",
                 "application/x-tcl",
                 "application/x-tex",
                 "application/x-texinfo",
                 "application/x-troff",
                 "application/x-troff-man",
                 "application/x-troff-me",
                 "application/x-troff-ms",
                 "application/x-ustar",
                 "application/x-wais-source",
                 "application/x-x509-ca-cert",
                 "application/x-xpinstall",
                 "application/x-zip-compressed",
                 "application/ynd.ms-pkipko",
                 "application/zip",
                 "audio/basic",
                 "audio/mid",
                 "audio/mpeg",
                 "audio/x-aiff",
                 "audio/x-mpegurl",
                 "audio/x-ms-wma",
                 "audio/x-pn-realaudio",
                 "audio/x-wav",
                 "image/bmp",
                 "image/bmpimage/x-bmp",
                 "image/cis-cod",
                 "image/gif",
                 "image/ief",
                 "image/jpeg",
                 "image/pipeg",
                 "image/png",
                 "image/svg+xml",
                 "image/tiff",
                 "image/x-cmu-raster",
                 "image/x-cmx",
                 "image/x-icon",
                 "image/x-portable-anymap",
                 "image/x-portable-bitmap",
                 "image/x-portable-graymap",
                 "image/x-portable-pixmap",
                 "image/x-rgb",
                 "image/x-xbitmap",
                 "image/x-xpixmap",
                 "image/x-xwindowdump",
                 "message/rfc822",
                 "text/css",
                 "text/h323",
                 "text/html",
                 "text/iuls",
                 "text/javascript",
                 "text/plain",
                 "text/richtext",
                 "text/scriptlet",
                 "text/tab-separated-values",
                 "text/vnd.wap.wml",
                 "text/webviewhtml",
                 "text/x-component",
                 "text/x-setext",
                 "text/x-vcard",
                 "video/mpeg",
                 "video/quicktime",
                 "video/x-la-asf",
                 "video/x-ms-asf",
                 "video/x-msvideo",
                 "video/x-sgi-movie",
                 "x-world/x-vrml")

    def __missing__(self, key):
        _key = key.split(';')[0].strip()
        if _key in self:
            return self[_key]

        log.warning("[MIMEHandler] Unknown MIME Type: %s", key)
        return self.passthrough

    def __init__(self):
        for mimetype in self.mimetypes:
            self[mimetype] = self.passthrough

        self.handlers = list()

        self.register_empty_handlers()
        self.register_fallback_handlers()
        self.register_zip_handlers()
        self.register_rar_handlers()
        self.register_pdf_handlers()
        self.register_android_handlers()
        self.register_java_jnlp_handlers()

    def register_empty_handlers(self):
        self['application/javascript']   = None
        self['application/x-javascript'] = None
        self['text/css']                 = None
        self['text/html']                = None
        # self['text/plain']             = None
        self['text/javascript']          = None

    def register_fallback_handlers(self):
        self['text/plain'] = self.handle_fallback

    def register_handler(self, mimetype, handler):
        self[mimetype] = handler
        self.handlers.append(handler)

    def register_zip_handlers(self):
        self.register_handler('application/zip', self.handle_zip)

    def register_rar_handlers(self):
        self.register_handler('application/x-rar-compressed', self.handle_rar)

    def register_pdf_handlers(self):
        if PEEPDF:
            self.register_handler('application/pdf', self.handle_pdf)

    def register_android_handlers(self):
        if ANDROGUARD:
            self['application/vnd.android.package-archive'] = self.handle_android

    def register_java_jnlp_handlers(self):
        self['application/x-java-jnlp-file'] = self.handle_java_jnlp

    def handle_fallback(self, url, content):
        for handler in self.handlers:
            try:
                if handler(url, content):
                    return True
            except:  # pylint:disable=bare-except
                pass

        return False

    def handle_zip(self, url, content):
        fp = StringIO(content)
        if not zipfile.is_zipfile(fp):
            return False

        try:
            zipdata = zipfile.ZipFile(fp)
        except:  # pylint:disable=bare-except
            return False

        for filename in zipdata.namelist():
            try:
                data = zipdata.read(filename)
            except:  # pylint:disable=bare-except
                continue

            sample = log.ThugLogging.log_file(data)
            if sample is None:
                continue

            try:
                md5 = sample['md5']
            except:  # pylint:disable=bare-except
                continue

            unzipped = os.path.join(log.ThugLogging.baseDir, 'unzipped')
            log.ThugLogging.store_content(unzipped, md5, data)

        return True

    def handle_rar(self, url, content):
        fd, rfile = tempfile.mkstemp()
        with open(rfile, 'wb') as fd:
            fd.write(content)

        try:
            rardata = rarfile.RarFile(rfile)
        except:  # pylint:disable=bare-except
            os.remove(rfile)
            return False

        for filename in rardata.namelist():
            try:
                data = rardata.read(filename)
            except:  # pylint:disable=bare-except
                continue

            sample = log.ThugLogging.log_file(data)
            if sample is None:
                continue

            try:
                md5 = sample['md5']
            except:  # pylint:disable=bare-except
                continue

            unzipped = os.path.join(log.ThugLogging.baseDir, 'unzipped')
            log.ThugLogging.store_content(unzipped, md5, data)

        os.remove(rfile)
        return True

    def getPeepXML(self, statsDict, url):
        """
            Slightly modified version of Peepdf getPeepXML function
        """

        root              = etree.Element('peepdf_analysis',
                                          url    = 'http://peepdf.eternal-todo.com',
                                          author = 'Jose Miguel Esparza')

        analysisDate      = etree.SubElement(root, 'date')
        analysisDate.text = datetime.today().strftime('%Y-%m-%d %H:%M')
        basicInfo         = etree.SubElement(root, 'basic')
        fileName          = etree.SubElement(basicInfo, 'filename')
        fileName.text     = statsDict['File']
        md5               = etree.SubElement(basicInfo, 'md5')
        md5.text          = statsDict['MD5']
        sha1              = etree.SubElement(basicInfo, 'sha1')
        sha1.text         = statsDict['SHA1']
        sha256            = etree.SubElement(basicInfo, 'sha256')
        sha256.text       = statsDict['SHA256']
        size              = etree.SubElement(basicInfo, 'size')
        size.text         = statsDict['Size']
        detection         = etree.SubElement(basicInfo, 'detection')

        if statsDict['Detection'] != [] and statsDict['Detection'] is not None:
            detectionRate        = etree.SubElement(detection, 'rate')
            detectionRate.text   = '%d/%d' % (statsDict['Detection'][0], statsDict['Detection'][1])
            detectionReport      = etree.SubElement(detection, 'report_link')
            detectionReport.text = statsDict['Detection report']

        version      = etree.SubElement(basicInfo, 'pdf_version')
        version.text = statsDict['Version']
        # binary       = etree.SubElement(basicInfo, 'binary', status = statsDict['Binary'].lower())
        # linearized   = etree.SubElement(basicInfo, 'linearized', status = statsDict['Linearized'].lower())
        encrypted    = etree.SubElement(basicInfo, 'encrypted', status = statsDict['Encrypted'].lower())

        if statsDict['Encryption Algorithms'] != []:
            algorithms = etree.SubElement(encrypted, 'algorithms')
            for algorithmInfo in statsDict['Encryption Algorithms']:
                algorithm      = etree.SubElement(algorithms, 'algorithm', bits = str(algorithmInfo[1]))
                algorithm.text = algorithmInfo[0]

        updates       = etree.SubElement(basicInfo, 'updates')
        updates.text  = statsDict['Updates']
        objects       = etree.SubElement(basicInfo, 'num_objects')
        objects.text  = statsDict['Objects']
        streams       = etree.SubElement(basicInfo, 'num_streams')
        streams.text  = statsDict['Streams']
        comments      = etree.SubElement(basicInfo, 'comments')
        comments.text = statsDict['Comments']
        errors        = etree.SubElement(basicInfo, 'errors', num = str(len(statsDict['Errors'])))

        for error in statsDict['Errors']:
            errorMessageXML      = etree.SubElement(errors, 'error_message')
            errorMessageXML.text = error

        advancedInfo = etree.SubElement(root, 'advanced')

        for version in range(len(statsDict['Versions'])):
            statsVersion = statsDict['Versions'][version]
            if version == 0:
                versionType = 'original'
            else:
                versionType = 'update'

            versionInfo = etree.SubElement(advancedInfo, 'version', num = str(version), type = versionType)
            catalog     = etree.SubElement(versionInfo, 'catalog')

            if statsVersion['Catalog']:
                catalog.set('object_id', statsVersion['Catalog'])

            info = etree.SubElement(versionInfo, 'info')
            if statsVersion['Info']:
                info.set('object_id', statsVersion['Info'])

            objects = etree.SubElement(versionInfo, 'objects', num = statsVersion['Objects'][0])

            for _id in statsVersion['Objects'][1]:
                _object = etree.SubElement(objects, 'object', id = str(_id))

                if statsVersion['Compressed Objects']:
                    if _id in statsVersion['Compressed Objects'][1]:
                        _object.set('compressed', 'true')
                    else:
                        _object.set('compressed', 'false')

                if statsVersion['Errors']:
                    if _id in statsVersion['Errors'][1]:
                        _object.set('errors', 'true')
                    else:
                        _object.set('errors', 'false')

            streams = etree.SubElement(versionInfo, 'streams', num = statsVersion['Streams'][0])

            for _id in statsVersion['Streams'][1]:
                stream = etree.SubElement(streams, 'stream', id = str(_id))

                if statsVersion['Xref Streams']:
                    if _id in statsVersion['Xref Streams'][1]:
                        stream.set('xref_stream', 'true')
                    else:
                        stream.set('xref_stream', 'false')

                if statsVersion['Object Streams']:
                    if _id in statsVersion['Object Streams'][1]:
                        stream.set('object_stream', 'true')
                    else:
                        stream.set('object_stream', 'false')

                if statsVersion['Encoded']:
                    if _id in statsVersion['Encoded'][1]:
                        stream.set('encoded', 'true')
                        if statsVersion['Decoding Errors']:
                            if _id in statsVersion['Decoding Errors'][1]:
                                stream.set('decoding_errors', 'true')
                            else:
                                stream.set('decoding_errors', 'false')
                    else:
                        stream.set('encoded', 'false')

            jsObjects = etree.SubElement(versionInfo, 'js_objects')

            if statsVersion['Objects with JS code']:
                for _id in statsVersion['Objects with JS code'][1]:
                    etree.SubElement(jsObjects, 'container_object', id = str(_id))

            actions    = statsVersion['Actions']
            events     = statsVersion['Events']
            vulns      = statsVersion['Vulns']
            elements   = statsVersion['Elements']
            suspicious = etree.SubElement(versionInfo, 'suspicious_elements')

            if events or actions or vulns or elements:
                if events:
                    triggers = etree.SubElement(suspicious, 'triggers')
                    for event in events:
                        trigger = etree.SubElement(triggers, 'trigger', name = event)
                        for _id in events[event]:
                            log.ThugLogging.log_exploit_event(url,
                                                              "Adobe Acrobat Reader",
                                                              "Adobe Acrobat Reader suspicious trigger: %s [object %s]" % (event, _id, )
                                                              )
                            etree.SubElement(trigger, 'container_object', id = str(_id))
                if actions:
                    actionsList = etree.SubElement(suspicious, 'actions')
                    for action in actions:
                        actionInfo = etree.SubElement(actionsList, 'action', name = action)
                        for _id in actions[action]:
                            log.ThugLogging.log_exploit_event(url,
                                                              "Adobe Acrobat Reader",
                                                              "Adobe Acrobat Reader suspicious action: %s [object %s]" % (action, _id, )
                                                              )
                            etree.SubElement(actionInfo, 'container_object', id = str(_id))

                if elements:
                    elementsList = etree.SubElement(suspicious, 'elements')
                    for element in elements:
                        elementInfo = etree.SubElement(elementsList, 'element', name = element)
                        if element in vulnsDict:
                            for vulnCVE in vulnsDict[element]:
                                if isinstance(vulnCVE, (list, tuple)):
                                    vulnCVE = ",".join(vulnCVE)

                                log.ThugLogging.log_exploit_event(url,
                                                                  "Adobe Acrobat Reader",
                                                                  "Adobe Acrobat Reader Exploit (%s)" % (vulnCVE, ),
                                                                  cve = vulnCVE)
                                cve = etree.SubElement(elementInfo, 'cve')
                                cve.text = vulnCVE
                        for _id in elements[element]:
                            etree.SubElement(elementInfo, 'container_object', id = str(_id))
                if vulns:
                    vulnsList = etree.SubElement(suspicious, 'js_vulns')
                    for vuln in vulns:
                        vulnInfo = etree.SubElement(vulnsList, 'vulnerable_function', name = vuln)
                        if vuln in vulnsDict:
                            for vulnCVE in vulnsDict[vuln]:
                                if isinstance(vulnCVE, (list, tuple)):
                                    vulnCVE = ",".join(vulnCVE)

                                log.ThugLogging.log_exploit_event(url,
                                                                  "Adobe Acrobat Reader",
                                                                  "Adobe Acrobat Reader Exploit (%s)" % (vulnCVE, ),
                                                                  cve = vulnCVE)
                                cve = etree.SubElement(vulnInfo, 'cve')
                                cve.text = vulnCVE
                        for _id in vulns[vuln]:
                            etree.SubElement(vulnInfo, 'container_object', id = str(_id))

            urls           = statsVersion['URLs']
            # suspiciousURLs = etree.SubElement(versionInfo, 'suspicious_urls')

            if urls:
                for url in urls:
                    urlInfo      = etree.SubElement(versionInfo, 'url')
                    urlInfo.text = url

        return etree.tostring(root, pretty_print = True)

    def swf_mastah(self, pdf, statsDict, url):
        """
            This code is taken from SWF Mastah by Brandon Dixon
        """
        swfdir = os.path.join(log.ThugLogging.baseDir, 'dropped', 'swf')
        count  = 0

        for version in range(len(statsDict['Versions'])):  # pylint:disable=unused-variable
            body = pdf.body[count]
            objs = body.objects

            for index in objs:
                # oid    = objs[index].id
                # offset = objs[index].offset
                # size   = objs[index].size
                details = objs[index].object

                if details.type in ("stream", ):
                    # encoded_stream = details.encodedStream
                    decoded_stream = details.decodedStream
                    header         = decoded_stream[:3]
                    is_flash       = [s for s in objs if header in ("CWS", "FWS")]

                    if is_flash:
                        data   = decoded_stream.strip()
                        sample = log.ThugLogging.log_file(data, url)
                        if sample is None:
                            continue

                        swffile = "%s.swf" % (sample["md5"], )
                        log.ThugLogging.store_content(swfdir, swffile, data)
                        log.warning("[PDF] Embedded SWF %s extracted from PDF %s", sample["md5"], statsDict["MD5"])

            count += 1

    def handle_pdf(self, url, content):
        sample = log.ThugLogging.build_sample(content, url)
        if sample is None or sample['type'] not in ('PDF', ):
            return

        fd, rfile = tempfile.mkstemp()
        with open(rfile, 'wb') as fd:
            fd.write(content)

        pdfparser = PDFParser()

        try:
            ret, pdf = pdfparser.parse(rfile, forceMode = True, looseMode = True)  # pylint:disable=unused-variable
        except:  # pylint:disable=bare-except
            os.remove(rfile)
            return False

        statsDict = pdf.getStats()
        analysis  = self.getPeepXML(statsDict, url)

        log_dir = os.path.join(log.ThugLogging.baseDir, "analysis", "pdf")
        log.ThugLogging.log_peepdf(log_dir, sample, analysis)

        self.swf_mastah(pdf, statsDict, url)
        os.remove(rfile)
        return True

    def do_build_apk_report(self, a):
        output = StringIO()

        a.get_files_types()

        output.write("[FILES] \n")
        for i in a.get_files():
            try:
                output.write("\t%s %s %x\n" % (i, a.files[i], a.files_crc32[i], ))
            except KeyError:
                output.write("\t%s %x\n" % (i, a.files_crc32[i], ))

        output.write("\n[PERMISSIONS] \n")
        details_permissions = a.get_details_permissions()
        for i in details_permissions:
            output.write("\t%s %s\n" % (i, details_permissions[i], ))

        output.write("\n[MAIN ACTIVITY]\n\t%s\n" % (a.get_main_activity(), ))

        output.write("\n[ACTIVITIES] \n")
        activities = a.get_activities()
        for i in activities:
            filters = a.get_intent_filters("activity", i)
            output.write("\t%s %s\n" % (i, filters or "", ))

        output.write("\n[SERVICES] \n")
        services = a.get_services()
        for i in services:
            filters = a.get_intent_filters("service", i)
            output.write("\t%s %s\n" % (i, filters or "", ))

        output.write("\n[RECEIVERS] \n")
        receivers = a.get_receivers()
        for i in receivers:
            filters = a.get_intent_filters("receiver", i)
            output.write("\t%s %s\n" % (i, filters or "", ))

        output.write("\n[PROVIDERS]\n\t%s\n\n" % (a.get_providers(), ))

        vm  = dvm.DalvikVMFormat(a.get_dex())
        vmx = analysis.uVMAnalysis(vm)

        output.write("Native code      : %s\n"   % (analysis.is_native_code(vmx), ))
        output.write("Dynamic code     : %s\n"   % (analysis.is_dyn_code(vmx), ))
        output.write("Reflection code  : %s\n"   % (analysis.is_reflection_code(vmx), ))
        output.write("ASCII Obfuscation: %s\n\n" % (analysis.is_ascii_obfuscation(vm), ))

        for i in vmx.get_methods():
            i.create_tags()
            if not i.tags.empty():
                output.write("%s %s %s\n" % (i.method.get_class_name(),
                                             i.method.get_name(),
                                             i.tags, ))

        return output

    def save_apk_report(self, sample, a, url):
        output  = self.do_build_apk_report(a)
        log_dir = os.path.join(log.ThugLogging.baseDir, "analysis", "apk")
        log.ThugLogging.log_androguard(log_dir, sample, output.getvalue())

    def build_apk_sample(self, data, url = None):
        sample = {
            "md5"   : hashlib.md5(data).hexdigest(),
            "sha1"  : hashlib.sha1(data).hexdigest(),
            "raw"   : data,
            "data"  : base64.b64encode(data),
            "type"  : "APK",
        }

        if SSDEEP:
            sample['ssdeep'] = ssdeep.hash(data)

        return sample

    def handle_android(self, url, content):
        ret = False

        fd, rfile = tempfile.mkstemp()
        with open(rfile, 'wb') as fd:
            fd.write(content)

        ret_type = androconf.is_android(rfile)

        if ret_type not in ("APK", ):
            os.remove(rfile)
            return ret

        try :
            a = apk.APK(rfile, zipmodule = 2)
            if a.is_valid_APK():
                sample = self.build_apk_sample(content, url)
                self.save_apk_report(sample, a, url)
                ret = True
        except:  # pylint:disable=bare-except
            pass

        os.remove(rfile)
        return ret

    @property
    def javaWebStartUserAgent(self):
        javaplugin = log.ThugVulnModules._javaplugin.split('.')
        last = javaplugin.pop()
        version = '%s_%s' % ('.'.join(javaplugin), last)
        return "JNLP/6.0 javaws/%s (b04) Java/%s" % (version, version, )

    def handle_java_jnlp(self, url, data):
        headers = dict()
        headers['Connection'] = 'keep-alive'

        try:
            soup = BeautifulSoup.BeautifulSoup(data, "lxml")
        except:  # pylint:disable=bare-except
            return

        jnlp = soup.find("jnlp")
        if jnlp is None:
            return

        codebase = jnlp.attrs['codebase'] if 'codebase' in jnlp.attrs else ''

        log.ThugLogging.add_behavior_warn(description = '[JNLP Detected]', method = 'Dynamic Analysis')

        jars = soup.find_all("jar")
        if not jars:
            return

        headers['User-Agent'] = self.javaWebStartUserAgent

        for jar in jars:
            try:
                url = "%s%s" % (codebase, jar.attrs['href'], )
                self.window._navigator.fetch(url, headers = headers, redirect_type = "JNLP")
            except:  # pylint:disable=bare-except
                pass

    def passthrough(self, url, data):
        """
        The method passthrough is the default handler associated to
        almost all Content-Types with the few exceptions defined in
        register_empty_handlers.
        """
        return True

    def get_handler(self, key):
        return self[key]



# Registry Pro (epRegPro.ocx)
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def DeleteKey(self, arg0, arg1):
    if arg0 in (80000001, 80000002, ):
        log.ThugLogging.add_behavior_warn('[RegistryPro ActiveX] Deleting [HKEY_LOCAL_MACHINE/%s]' % (arg1, ))
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "RegistryPro ActiveX",
                                          "Deleting Regkey",
                                          forward = False,
                                          data = {
                                              "regkey": "HKEY_LOCAL_MACHINE/" + str(arg1)
                                          })


def About(self):
    log.ThugLogging.add_behavior_warn('[RegistryPro ActiveX] About called')



#!/usr/bin/env python
#
# SessionStorage.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .Storage import Storage


class SessionStorage(Storage):
    def __init__(self, *args, **kwargs):
        super(SessionStorage, self).__init__(*args, **kwargs)

    def __repr__(self, _repr_running = None):
        return "sessionStorage"



#!/usr/bin/env python

from thug.DOM.JSClass import JSClass


class DOMException(RuntimeError, JSClass):
    def __init__(self, code):
        self.code = code

    # ExceptionCode
    INDEX_SIZE_ERR                 = 1   # If index or size is negative, or greater than the allowed value
    DOMSTRING_SIZE_ERR             = 2   # If the specified range of text does not fit into a DOMString
    HIERARCHY_REQUEST_ERR          = 3   # If any node is inserted somewhere it doesn't belong
    WRONG_DOCUMENT_ERR             = 4   # If a node is used in a different document than the one that created it (that doesn't support it)
    INVALID_CHARACTER_ERR          = 5   # If an invalid or illegal character is specified, such as in a name.
    NO_DATA_ALLOWED_ERR            = 6   # If data is specified for a node which does not support data
    NO_MODIFICATION_ALLOWED_ERR    = 7   # If an attempt is made to modify an object where modifications are not allowed
    NOT_FOUND_ERR                  = 8   # If an attempt is made to reference a node in a context where it does not exist
    NOT_SUPPORTED_ERR              = 9   # If the implementation does not support the type of object requested
    INUSE_ATTRIBUTE_ERR            = 10  # If an attempt is made to add an attribute that is already in use elsewhere

    # Introduced in Level 2
    INVALID_STATE_ERR              = 11  # If an attempt is made to use an object that is not, or is no longer, usable
    SYNTAX_ERR                     = 12  # If an invalid or illegal string is specified
    INVALID_MODIFICATION_ERR       = 13  # If an attempt is made to modify the type of the underlying object
    NAMESPACE_ERR                  = 14  # If an attempt is made to create or change an object in a way which is incorrect with regards to namespaces
    INVALID_ACCESS_ERR             = 15  # If a parameter or an operation is not supported by the underlying object



# Clever Internet ActiveX Suite 6.2 (CLINETSUITEX6.OCX) Arbitrary file download/overwrite Exploit

import logging

log = logging.getLogger("Thug")


def GetToFile(self, url, _file):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Clever Internet ActiveX Suite 6.2 (CLINETSUITEX6.OCX)",
                                      "Arbitrary File Download/Overwrite Exploit",
                                      data = {
                                                "url" : url,
                                                "file": _file
                                             }
                                     )



#!/usr/bin/env python

import six
import bs4 as BeautifulSoup
from .DOMImplementation import DOMImplementation


def getDOMImplementation(dom = None, **kwds):
    return DOMImplementation(dom if dom else BeautifulSoup.BeautifulSoup(), **kwds)


def parseString(html, **kwds):
    return DOMImplementation(BeautifulSoup.BeautifulSoup(html, "html.parser"), **kwds)


def parse(_file, **kwds):
    if isinstance(_file, six.string_types):
        with open(_file, 'r') as f:
            return parseString(f.read())

    return parseString(_file.read(), **kwds)



#!/usr/bin/env python

from Node import Node


class EntityReference(Node):
    def __init__(self, doc, name):
        self.name = name
        Node.__init__(self, doc)

    @property
    def nodeName(self):
        return self.name

    @property
    def nodeType(self):
        return Node.ENTITY_REFERENCE_NODE

    @property
    def nodeValue(self):
        return None




import logging

log = logging.getLogger("Thug")


def GetVariable(self, arg):
    if arg in ("$version", ):
        version = ['0', '0', '0', '0']
        idx     = 0

        for p in log.ThugVulnModules.shockwave_flash.split('.'):
            version[idx] = p
            idx += 1

        return "WIN %s" % (','.join(version))



# VLC ActiveX Control
# CVE-2007-4619, CVE-2007-6262

import logging

log = logging.getLogger("Thug")


def getVariable(self, arg):
    if len(arg) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "VLC ActiveX",
                                          "getVariable Overflow",
                                          cve = "CVE-2007-6262")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6262", None)


def setVariable(self, arg0, arg1):
    if len(arg0) > 255 or len(arg1) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "VLC ActiveX",
                                          "setVariable Overflow",
                                          cve = "CVE-2007-6262")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6262", None)

def addTarget(self, arg0, arg1, arg2, arg3):
    if len(arg0) > 255 or len(arg1) > 255 or len(arg2) > 255 or len(arg3) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "VLC ActiveX",
                                          "addTarget Overflow",
                                          cve = "CVE-2007-6262")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6262", None)



#!/usr/bin/env python

import six
import logging

log = logging.getLogger("Thug")

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

try:
    from io import StringIO
except ImportError:
    try:
        from cStringIO import StringIO
    except ImportError:
        from StringIO import StringIO

import bs4 as BeautifulSoup

from thug.DOM.W3C.Document import Document
from thug.DOM.W3C.DocumentCompatibleInfoCollection import DocumentCompatibleInfoCollection
from .HTMLCollection import HTMLCollection
from .HTMLElement import HTMLElement
from .HTMLBodyElement import HTMLBodyElement
from .HTMLAllCollection import HTMLAllCollection
from .HTMLDocumentCompatibleInfo import HTMLDocumentCompatibleInfo
from .text_property import text_property
from .xpath_property import xpath_property


class HTMLDocument(Document):
    title       = xpath_property("/html/head/title/text()")
    # body        = xpath_property("/html/body[1]", readonly = True)
    images      = xpath_property("//img", readonly = True)
    applets     = xpath_property("//applet", readonly = True)
    # forms       = xpath_property("//form", readonly = True)
    links       = xpath_property("//a[@href]", readonly = True)
    anchors     = xpath_property("//a[@name]", readonly = True)
    innerHTML   = text_property()

    def __init__(self, doc, win = None, referer = None, lastModified = None, cookie = ''):
        Document.__init__(self, doc)

        self._win           = win
        self._referer       = referer
        self._lastModified  = lastModified
        self._cookie        = cookie
        self._html          = None
        self._readyState    = "loading"
        self._domain        = urlparse(self._win.url).hostname if self._win else ''
        self.current        = None
        self.__init_personality()

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        if log.ThugOpts.Personality.browserMajorVersion < 8:
            self._compatible = None
        else:
            self._compatible = DocumentCompatibleInfoCollection(self.doc, [])

        if log.ThugOpts.Personality.browserMajorVersion < 11:
            self.all = self._all
            self.createStyleSheet = self._createStyleSheet

    def __init_personality_Firefox(self):
        pass

    def __init_personality_Chrome(self):
        self.all = self._all

    def __init_personality_Safari(self):
        self.all = self._all

    def __init_personality_Opera(self):
        pass

    def __getattr__(self, attr):
        if attr in ('getBoxObjectFor', ) and not log.ThugOpts.Personality.isFirefox():
            raise AttributeError

        if self._win and getattr(self._win, "doc", None):
            if attr in self._win.doc.DFT.handled_on_events:
                return None

            if attr in self._win.doc.DFT._on_events:
                return None

        _attr = self.getElementById(attr)
        if _attr:
            return _attr

        log.info("[HTMLDocument] Undefined: %s", attr)
        raise AttributeError

    def getWindow(self):
        return self._win

    def setWindow(self, win):
        self._win = win

    window = property(getWindow, setWindow)

    @property
    def body(self):
        tag = self.doc.find('body')
        return HTMLBodyElement(self.doc, tag if tag else self.doc)

    @property
    def tag(self):
        return self

    @property
    def _node(self):
        return self

    @property
    def referrer(self):
        if self._referer:
            return str(self._referer)

        return ""

    @property
    def forms(self):
        from thug.DOM.W3C.DOMImplementation import DOMImplementation
        return HTMLCollection(self.doc, [DOMImplementation.createHTMLElement(self.doc, f) for f in self.doc.find_all('form')])

    @property
    def styleSheets(self):
        from thug.DOM.W3C.DOMImplementation import DOMImplementation
        return HTMLCollection(self.doc, [DOMImplementation.createHTMLElement(self.doc, f) for f in self.doc.find_all('style')])

    @property
    def lastModified(self):
        return self._lastModified

    def getCookie(self):
        return self._cookie

    def setCookie(self, value):
        self._cookie = value

    cookie = property(getCookie, setCookie)

    def getDomain(self):
        return self._domain

    def setDomain(self, value):
        self._domain = value

    domain = property(getDomain, setDomain)

    @property
    def URL(self):
        return self._win.url if self._win else ''

    @property
    def documentElement(self):
        return HTMLElement(self, self.doc.find('html'))

    # FIXME
    @property
    def readyState(self):
        return self._readyState

    @property
    def compatMode(self):
        return "CSS1Compat"

    def getCompatible(self):
        return self._compatible

    def setCompatible(self, compatible):
        _compatibles = list()

        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion >= 8:
            for s in compatible.split(';'):
                try:
                    (useragent, version) = s.split('=')
                except ValueError:
                    # Ignore the http-equiv X-UA-Compatible content if its
                    # format is not correct
                    return

                for v in version.split(','):
                    p = HTMLDocumentCompatibleInfo(useragent, v)
                    _compatibles.append(p)

            self._compatible = DocumentCompatibleInfoCollection(self.doc, _compatibles)

    compatible = property(getCompatible, setCompatible)

    @property
    def documentMode(self):
        # version = log.ThugOpts.Personality.browserVersion
        major   = log.ThugOpts.Personality.browserMajorVersion

        if major < 8:
            return 7 if self.compatMode in ("CSS1Compat", ) else 5

        self.window.doc.DFT.force_handle_meta_x_ua_compatible()

        engine = 0

        for index in range(self.compatible.length):
            item = self.compatible.item(index)
            if not item.userAgent.lower() in ("ie", ):
                continue

            _version = item.version.lower()
            if _version in ('edge', ):
                engine = major
                break

            mode_version = _version

            if _version.startswith('emulateie'):
                mode_version = _version.split("emulateie")[1]

            try:
                mode_version = int(mode_version)
            except ValueError:
                continue

            if mode_version not in (5, 7, 8, 9, 10):
                continue

            if mode_version <= major and mode_version >= engine:
                engine = mode_version

        if not engine:
            engine = min(major, 10)

        return engine

    def open(self, mimetype = 'text/html', replace = False):
        self._html = StringIO()

        return self

    def close(self):
        if self._html is None:
            return

        html = self._html.getvalue()
        self._html.close()
        self._html = None

        self.doc = BeautifulSoup.BeautifulSoup(html, "html5lib")

    def write(self, html):
        if self._html:
            self._html.write(unicode(html))
            return

        tag  = self.current
        body = self.doc.find('body')

        if tag.parent is None:
            parent = body
        else:
            parent = body if body and tag.parent.name in ('html', ) else tag.parent

        if isinstance(html, six.integer_types):
            html = unicode(html)

        for t in BeautifulSoup.BeautifulSoup(html, "html.parser").contents:
            if isinstance(t, six.string_types):
                child = list(parent.children)[-1]

                if isinstance(child, BeautifulSoup.NavigableString):
                    child.string.replace_with(child.string + t)
                if isinstance(child, BeautifulSoup.Tag):
                    child.append(t)

            if isinstance(t, BeautifulSoup.Tag):
                parent.insert(len(parent.contents), t)

            name = getattr(t, "name", None)
            if name in ('script', None):
                continue

            try:
                handler = getattr(self._win.doc.DFT, "handle_%s" % (name, ), None)
            except:  # pylint:disable=bare-except
                handler = getattr(log.DFT, "handle_%s" % (name, ), None)

            if handler:
                handler(t)

    def writeln(self, text):
        self.write(text + "\n")

    # DOM Level 2 moves getElementbyId in Document object inherited by
    # HTMLDocument
    #
    # def getElementById(self, elementId):
    #    tag = self.doc.find(id = elementId)
    #    return DOMImplementation.createHTMLElement(self.doc, tag) if tag else None

    def getElementsByName(self, elementName):
        tags = self.doc.find_all(attrs = {'name': elementName})

        return HTMLCollection(self.doc, tags)

    @property
    def _all(self):
        s = [p for p in self.doc.find_all(text = False)]
        return HTMLAllCollection(self.doc, s)

    def _createStyleSheet(self, URL = None, index = None):
        # Creates a styleSheet object and inserts it into the current document.
        # The createStyleSheet method is only supported by Internet Explorer.
        # In other browsers, use the createElement method to create a new link
        # or style element, and the insertBefore or appendChild method to insert
        # it into the document.
        #
        # URL Optional. String that specifies the URL of a style file. If an
        # URL is specified, a new link element is inserted into the current
        # document that imports the style file. If this parameter is not specified
        # or an empty string is set, then an empty style element is inserted into
        # the current document.
        #
        # index Optional. Integer that specifies the position of the new styleSheet
        # object in the styleSheets collection of the document. This parameter
        # also has effect on the source position of the inserted link or style
        # element. If this parameter is not specified, the new styleSheet object
        # is inserted at the end of the styleSheets collection and the new link or
        # style element is inserted after the other link and style elements in
        # source order.

        _type = 'style' if not URL else 'link'

        obj = self.createElement(_type)
        if _type in ('link', ):
            obj.href = URL
            obj.rel = "stylesheet"

        head = self.getElementsByTagName("head")[0]
        if head is None:
            return obj

        if index is None:
            head.appendChild(obj)
            return obj

        length  = len(head.childNodes)
        matches = 0
        pos     = 0

        while pos < length:
            if head.childNodes[pos].tagName.lower() in (_type, ):
                matches += 1

            if matches > index:
                head.insertBefore(obj, head.childNodes[pos])
                break

            pos += 1

        if matches <= index:
            head.appendChild(obj)

        return obj




import logging

log = logging.getLogger("Thug")


def definition(self, arg):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "MSXML2.DOMDocument",
                                      "Microsoft XML Core Services MSXML Uninitialized Memory Corruption",
                                      cve = "CVE-2012-1889")  # pylint:disable=undefined-variable

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2012-1889", None)



#!/usr/bin/env python

import bs4 as BeautifulSoup

from .Node import Node


class Attr(Node):
    _value = ""

    def __init__(self, doc, parent, attr):
        self.doc    = doc
        self.parent = parent
        self.attr   = attr
        self.tag    = BeautifulSoup.Tag(parser = self.doc, name = 'attr')
        Node.__init__(self, doc)

        self._specified = False
        self._value     = self.getValue()

    def __repr__(self):
        return "<Attr object %s%s at 0x%08X>" % ("%s." % self.parent.tagName if self.parent else "", self.attr, id(self))

    def __eq__(self, other):
        return hasattr(other, "parent") and self.parent == other.parent and \
               hasattr(other, "attr") and self.attr == other.attr

    @property
    def nodeType(self):
        return Node.ATTRIBUTE_NODE

    @property
    def nodeName(self):
        return self.attr

    def getNodeValue(self):
        return self.getValue()

    def setNodeValue(self, value):
        return self.setValue(value)

    nodeValue = property(getNodeValue, setNodeValue)

    @property
    def childNodes(self):
        from .NodeList import NodeList

        return NodeList(self.parent.doc, [])

    @property
    def parentNode(self):
        return self.parent

    # Introduced in DOM Level 2
    @property
    def ownerElement(self):
        if self.parent:
            if self.parent.nodeType == Node.ELEMENT_NODE:
                return self.parent

        return None

    @property
    def ownerDocument(self):
        return self.parent.doc

    @property
    def name(self):
        return self.attr

    @property
    def specified(self):
        if self.ownerElement is None:
            return True

        return self._specified

    def getValue(self):
        if self.parent:
            if self.parent.tag.has_attr(self.attr):
                self._specified = True
                return self.parent.tag[self.attr]

        return self._value

    def setValue(self, value):
        self._value = value

        if self.parent:
            self._specified = True
            self.parent.tag[self.attr] = value

    value = property(getValue, setValue)



#!/usr/bin/env python
#
# BaseClassifier.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import yara
import logging

log = logging.getLogger("Thug")


class BaseClassifier(object):
    def __init__(self):
        self.matches = list()
        self.init_rules()
        self.init_filters()

    def init_rules(self):
        self._rules = dict()
        self.rules_namespace_id = 1

        p = getattr(self, 'default_rule_file', None)
        if p is None:
            log.warn("[%s] Skipping not existing default classification rule file", self.classifier)
            return

        r = os.path.join(log.configuration_path, p)

        if not os.path.exists(r):
            log.warn("[%s] Skipping not existing default classification rule file", self.classifier)
            return

        self._rules['namespace0'] = r
        self.rules = yara.compile(filepaths = self._rules)

    def init_filters(self):
        self._filters = dict()
        self.filters_namespace_id = 1

        p = getattr(self, 'default_filter_file', None)
        if p is None:
            log.warn("[%s] Skipping not existing default filter file", self.classifier)
            return

        r = os.path.join(log.configuration_path, p)

        if not os.path.exists(r):
            log.warn("[%s] Skipping not existing default filter file", self.classifier)
            return

        self._filters['namespace0'] = r
        self.filters = yara.compile(filepaths = self._filters)

    def add_rule(self, rule_file):
        if not os.path.exists(rule_file):
            log.warn("[%s] Skipping not existing classification rule file %s", self.classifier, rule_file)
            return

        self._rules["namespace{}".format(self.rules_namespace_id)] = rule_file
        self.rules_namespace_id += 1
        self.rules = yara.compile(filepaths = self._rules)

    def add_filter(self, filter_file):
        if not os.path.exists(filter_file):
            log.warn("[%s] Skipping not existing filter file %s", self.classifier, filter_file)
            return

        self._filters["namespace{}".format(self.filters_namespace_id)] = filter_file
        self.filters_namespace_id += 1
        self.filters = yara.compile(filepaths = self._filters)

    @property
    def result(self):
        return self.matches



# Lycos FileUploader Module 2.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetHandwriterFilename(self, val):
    self.__dict__['HandwriterFilename'] = val

    if len(val) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Lycos FileUploader ActiveX",
                                          "Overflow in HandwriterFilename property")



#!/usr/bin/env python

import ez_setup
ez_setup.use_setuptools()

import os
import glob
from setuptools import setup, find_packages

os.environ['BUILD_LIB'] = '1'

import thug

personalities_path = os.path.join(thug.__configuration_path__, "personalities")
rules_path         = os.path.join(thug.__configuration_path__, "rules")
scripts_path       = os.path.join(thug.__configuration_path__, "scripts")
plugins_path       = os.path.join(thug.__configuration_path__, "plugins")
hooks_path         = os.path.join(thug.__configuration_path__, "hooks")

html_rules_path    = os.path.join(rules_path, "htmlclassifier")
js_rules_path      = os.path.join(rules_path, "jsclassifier")
vbs_rules_path     = os.path.join(rules_path, "vbsclassifier")
url_rules_path     = os.path.join(rules_path, "urlclassifier")
sample_rules_path  = os.path.join(rules_path, "sampleclassifier")
html_filter_path   = os.path.join(rules_path, "htmlfilter")
js_filter_path     = os.path.join(rules_path, "jsfilter")
vbs_filter_path    = os.path.join(rules_path, "vbsfilter")
url_filter_path    = os.path.join(rules_path, "urlfilter")
sample_filter_path = os.path.join(rules_path, "samplefilter")


setup(
    name = "thug",
    version = thug.__version__,
    author = "Angelo Dell'Aera",
    author_email = "angelo.dellaera@honeynet.org",
    description = "Low-interaction honeyclient Thug",
    license = "GPLv2",
    long_description = open("README.rst").read(),
    url = "http://buffer.github.io/thug/",
    download_url = "https://github.com/buffer/thug/",
    platforms = ["Linux", ],
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Topic :: Security",
    ],
    package_data = {
           ""       : ["*.js"],
           },
    packages = find_packages(),
    data_files = [
        (thug.__configuration_path__, ["thug/Analysis/honeyagent/honeyagent.conf.sample",
                                       "thug/Analysis/virustotal/virustotal.conf.default",
                                       "thug/Logging/logging.conf.default"]),
        (personalities_path         , glob.glob("thug/DOM/personalities/*.json")),
        (rules_path                 , glob.glob("thug/Classifier/rules/*.yar")),
        (scripts_path               , ["thug/DOM/thug.js",
                                       "thug/DOM/storage.js",
                                       "thug/Debugger/d8.js"]),
        (plugins_path               , []),
        (hooks_path                 , []),
        (html_rules_path            , glob.glob("thug/Classifier/rules/htmlclassifier/*.yar")),
        (js_rules_path              , glob.glob("thug/Classifier/rules/jsclassifier/*.yar")),
        (vbs_rules_path             , glob.glob("thug/Classifier/rules/vbsclassifier/*.yar")),
        (url_rules_path             , glob.glob("thug/Classifier/rules/urlclassifier/*.yar")),
        (sample_rules_path          , glob.glob("thug/Classifier/rules/sampleclassifier/*.yar")),
        (html_filter_path           , glob.glob("thug/Classifier/rules/htmlfilter/*.yar")),
        (js_filter_path             , glob.glob("thug/Classifier/rules/jsfilter/*.yar")),
        (vbs_filter_path            , glob.glob("thug/Classifier/rules/vbsfilter/*.yar")),
        (url_filter_path            , glob.glob("thug/Classifier/rules/urlfilter/*.yar")),
        (sample_filter_path         , glob.glob("thug/Classifier/rules/samplefilter/*.yar")),

    ],
    install_requires = open("requirements.txt").read().splitlines(),
    entry_points = {
        "console_scripts": [
            "thug = thug.thug:main",
        ]
    }
)



#!/usr/bin/env python
#
# SampleLogging.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import logging
import base64
import tempfile
import hashlib
import zipfile
import pefile

SSDEEP = True

try:
    import ssdeep
except ImportError:
    SSDEEP = False

log = logging.getLogger("Thug")


class SampleLogging(object):
    def __init__(self):
        self.types = ('PE',
                      'PDF',
                      'JAR',
                      'SWF', )

    def is_pe(self, data):
        try:
            pefile.PE(data = data, fast_load = True)
        except:  # pylint:disable=bare-except
            return False

        return True

    def get_imphash(self, data):
        try:
            pe = pefile.PE(data = data)
        except:  # pylint:disable=bare-except
            return None

        return pe.get_imphash()

    def is_pdf(self, data):
        return (data[:1024].find('%PDF') != -1)

    def is_jar(self, data):
        fd, jar = tempfile.mkstemp()
        with open(jar, 'wb') as fd:
            fd.write(data)

        try:
            z = zipfile.ZipFile(jar)
            if [t for t in z.namelist() if t.endswith('.class')]:
                os.remove(jar)
                return True
        except:  # pylint:disable=bare-except
            pass

        os.remove(jar)
        return False

    def is_swf(self, data):
        return data.startswith('CWS') or data.startswith('FWS')

    def get_sample_type(self, data):
        for t in self.types:
            p = getattr(self, 'is_%s' % (t.lower(), ), None)
            if p and p(data):
                return t

        return None

    def build_sample(self, data, url = None, sampletype = None):
        if not data:
            return None

        p = dict()

        if sampletype:
            p['type'] = sampletype
        else:
            p['type'] = self.get_sample_type(data)

        if p['type'] is None:
            return None

        p['md5']  = hashlib.md5(data).hexdigest()
        p['sha1'] = hashlib.sha1(data).hexdigest()

        if SSDEEP:
            p['ssdeep'] = ssdeep.hash(data)

        if p['type'] in ('PE', ):
            imphash = self.get_imphash(data)
            if imphash:
                p['imphash'] = imphash

        if url:
            p['url'] = url

        p['data'] = base64.b64encode(data)

        return p



#!/usr/bin/env python
#
# MimeTypes.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
from .MimeType import MimeType
from .Plugin import Plugin

log = logging.getLogger("Thug")


class MimeTypes(dict):
    def __init__(self):
        if not log.ThugVulnModules.acropdf_disabled:
            self['application/pdf'] = MimeType({
                                            'description'   : 'Adobe Acrobat Plug-In',
                                            'suffixes'      : 'pdf',
                                            'filename'      : 'npctrl.dll',
                                            'type'          : 'application/pdf',
                                            'enabledPlugin' : Plugin({'name'        : 'Adobe Acrobat',
                                                                      'version'     : '%s' % (log.ThugVulnModules.acropdf_pdf, ),
                                                                      'description' : 'Adobe Acrobat Plug-In'}),
                                            'enabled'       : True})

        if not log.ThugVulnModules.shockwave_flash_disabled:
            self['application/x-shockwave-flash'] = MimeType({
                                            'description'   : 'Shockwave Flash',
                                            'suffixes'      : 'swf',
                                            'filename'      : 'Flash32_%s.ocx' % ('_'.join(log.ThugVulnModules.shockwave_flash.split('.')), ),
                                            'type'          : 'application/x-shockwave-flash',
                                            'enabledPlugin' : Plugin({'name'        : 'Shockwave Flash %s' % (log.ThugVulnModules.shockwave_flash, ),
                                                                      'version'     : '%s' % (log.ThugVulnModules.shockwave_flash, ),
                                                                      'description' : 'Shockwave Flash %s' % (log.ThugVulnModules.shockwave_flash, ), }),
                                            'enabled'       : True})

        if not log.ThugOpts.Personality.isIE():
            if not log.ThugVulnModules.javaplugin_disabled:
                self['application/x-java-applet'] = MimeType({
                                                              'description'   : 'Java Applet',
                                                              'suffixes'      : 'jar',
                                                              'filename'      : 'npjp2.dll',
                                                              'type'          : 'application/x-java-applet;jpi-version=%s' % (log.ThugVulnModules._javaplugin, ),
                                                              'enabledPlugin' : Plugin({'name'        : 'Java %s' % (log.ThugVulnModules._javaplugin, ),
                                                                                        'version'     : '%s' % (log.ThugVulnModules._javaplugin, ),
                                                                                        'description' : 'Java'}),
                                                              'enabled'       : True})

        if log.ThugOpts.Personality.isWindows():
            self['application/x-ms-wmz'] = MimeType({
                                                'description'   : 'Windows Media Player',
                                                'suffixes'      : 'wmz',
                                                'filename'      : 'npdsplay.dll',
                                                'type'          : 'application/x-ms-wmz',
                                                'enabledPlugin' : Plugin({'name'        : 'Windows Media Player 7',
                                                                          'version'     : '7',
                                                                          'description' : 'Windows Media Player 7', }),
                                                'enabled'       : True})

    def __getitem__(self, key):
        try:
            key = int(key)
            return self.item(key)
        except ValueError:
            return dict.__getitem__(self, key) if key in self else MimeType()

    @property
    def length(self):
        return len(self)

    def item(self, index):
        if index >= self.length:
            return MimeType()

        return list(self.values())[index]

    def namedItem(self, key):
        return dict.__getitem__(self, key) if key in self else MimeType()



# HP Info Center ActiveX Control
# CVE-2007-6331, CVE-2007-6332, CVE-2007-6333

import logging

log = logging.getLogger("Thug")


def LaunchApp(self, prog, args, unk):
    log.ThugLogging.add_behavior_warn("[HP Info Center ActiveX] LaunchApp called to run: %s %s" % (prog, args, ), "CVE-2007-6331")
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "HP Info Center ActiveX",
                                      "LaunchApp called to run",
                                      cve = 'CVE-2007-6331',
                                      forward = False,
                                      data = {
                                          "command": prog,
                                          "args"   : args
                                      }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6331", None)


def SetRegValue(self, key, section, keyname, value):
    log.ThugLogging.add_behavior_warn("[HP Info Center ActiveX] SetRegValue: %s/%s/%s set to %s" % (str(key),
                                                                                                    str(section),
                                                                                                    str(keyname),
                                                                                                    str(value), ),
                                      "CVE-2007-6332")

    log.ThugLogging.log_exploit_event(self._window.url,
                                      "HP Info Center ActiveX",
                                      "SetRegValue",
                                      cve = 'CVE-2007-6332',
                                      forward = False,
                                      data = {
                                          "key"    : str(key),
                                          "section": str(section),
                                          "keyname": str(keyname),
                                          "value"  : str(value)
                                      }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6332", None)


def GetRegValue(self, key, section, keyname):
    log.ThugLogging.add_behavior_warn("[HP Info Center ActiveX] GetRegValue, reading: %s/%s/%s" % (str(key),
                                                                                                   str(section),
                                                                                                   str(keyname), ),
                                      "CVE-2007-6333")

    log.ThugLogging.log_exploit_event(self._window.url,
                                      "HP Info Center ActiveX",
                                      "GetRegValue",
                                      cve = 'CVE-2007-6333',
                                      forward = False,
                                      data = {
                                          "key": str(key),
                                          "section": str(section),
                                          "keyname": str(keyname)
                                      }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6333", None)


def EvaluateRules(self):
    log.ThugLogging.log_exploit_event(self._window.url, "HP Info Center ActiveX", "EvaluateRules")


def SaveToFile(self, path):
    log.ThugLogging.add_behavior_warn("[HP Info Center ActiveX] SaveToFile(), writes to %s" % (path, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "HP Info Center ActiveX",
                                      "SaveToFile",
                                      data = {
                                          "filename": path
                                      },
                                      forward = False)


def ProcessRegistryData(self, parm):
    log.ThugLogging.add_behavior_warn("[HP Info Center ActiveX] ProcessRegistryData: %s " % (parm, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "HP Info Center ActiveX",
                                      "ProcessRegistryData",
                                      data = {
                                          "param": parm
                                      },
                                      forward = False)



#!/usr/bin/env python
#
# ActiveX.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

# import new
import logging
from .CLSID import CLSID

log = logging.getLogger("Thug")

acropdf   = ( 'acropdf.pdf',
              'pdf.pdfctrl',
              'CA8A9780-280D-11CF-A24D-444553540000', )

shockwave = ( 'shockwaveflash.shockwaveflash',
              'shockwaveflash.shockwaveflash.9',
              'shockwaveflash.shockwaveflash.10',
              'swctl.swctl',
              'swctl.swctl.8',
              '233C1507-6A77-46A4-9443-F871F945D258', )

java_deployment_toolkit = ( 'CAFEEFAC-DEC7-0000-0000-ABCDEFFEDCBA',
                            '8AD9C840-044E-11D1-B3E9-00805F499D93', )


class _ActiveXObject(object):
    shockwave_flash = { 'shockwaveflash.shockwaveflash'    : '10',
                        'shockwaveflash.shockwaveflash.9'  : '9' ,
                        'shockwaveflash.shockwaveflash.10' : '10',
                        'shockwaveflash.shockwaveflash.11' : '11',
                        'shockwaveflash.shockwaveflash.12' : '12'}

    def __init__(self, window, cls, typename = 'name'):
        self.funcattrs = dict()
        self._window   = window
        obj            = None
        methods        = dict()
        self.shockwave = log.ThugVulnModules.shockwave_flash.split('.')[0]
        self.cls       = cls

        if typename == 'id':
            if len(cls) > 5 and cls[:6].lower() == 'clsid:':
                cls = cls[6:].upper()

            if cls.startswith('{') and cls.endswith('}'):
                cls = cls[1:-1]

        if typename == 'name':
            cls = cls.lower()

        # Adobe Acrobat Reader
        if cls in acropdf and log.ThugVulnModules.acropdf_disabled:
            log.warning("Unknown ActiveX Object: %s", cls)
            raise TypeError()

        # Shockwave Flash
        if cls in shockwave and log.ThugVulnModules.shockwave_flash_disabled:
            log.warning("Unknown ActiveX Object: %s", cls)
            raise TypeError()

        if cls in self.shockwave_flash and self.shockwave not in (self.shockwave_flash[cls], ):
            log.warning("Unknown ActiveX Object: %s", cls)
            raise TypeError()

        _cls = cls

        # Java Deployment Toolkit
        if cls in java_deployment_toolkit and log.ThugVulnModules.javaplugin_disabled:
            log.warning("Unknown ActiveX Object: %s", cls)
            raise TypeError()

        # JavaPlugin
        if cls.lower().startswith('javaplugin'):
            if log.ThugVulnModules.javaplugin_disabled or not cls.endswith(log.ThugVulnModules.javaplugin):
                log.warning("Unknown ActiveX Object: %s", cls)
                raise TypeError()
            else:
                _cls = 'javaplugin'

        # JavaWebStart
        if cls.lower().startswith('javawebstart.isinstalled'):
            if log.ThugVulnModules.javaplugin_disabled or not cls.endswith(log.ThugVulnModules.javawebstart_isinstalled):
                log.warning("Unknown ActiveX Object: %s", cls)
                raise TypeError()
            else:
                _cls = 'javawebstart.isinstalled'

        for c in CLSID:
            if _cls in c[typename]:
                obj = c
                break

        if not obj:
            log.warning("Unknown ActiveX Object: %s", cls)
            raise TypeError()

        log.warning("ActiveXObject: %s", cls)

        for method_name, method in obj['methods'].items():
            # _method = new.instancemethod(method, self, _ActiveXObject)
            _method = method.__get__(self, _ActiveXObject)
            setattr(self, method_name, _method)
            methods[method] = _method

        for attr_name, attr_value in obj['attrs'].items():
            setattr(self, attr_name, attr_value)

        for attr_name, attr_value in obj['funcattrs'].items():
            self.funcattrs[attr_name] = methods[attr_value]

        if cls.lower() in ('wscript.shell', ) and (not hasattr(window, 'WScript') or window.WScript is None):
            window.WScript = self

    def __setattr__(self, name, value):
        self.__dict__[name] = value

        if name in self.funcattrs:
            self.funcattrs[name](value)

    def __getattr__(self, name):
        for key, value in self.__dict__.items():
            if key.lower() == name.lower():
                return value

        if name in self.funcattrs:
            value = self.funcattrs[name]()
            self.__dict__[name] = value
            return value

        if name not in ('__watchpoints__'):
            log.warning("Unknown ActiveX Object (%s) attribute: %s", self.cls, name)

        raise AttributeError


def register_object(s, clsid):
    funcattrs = dict()  # pylint:disable=unused-variable
    methods   = dict()
    obj       = None

    if not clsid.startswith('clsid:'):
        log.warning("Unknown ActiveX object: %s", clsid)
        return None

    clsid = clsid[6:].upper()
    if clsid.startswith('{') and clsid.endswith('}'):
        clsid = clsid[1:-1]

    # Adobe Acrobat Reader
    if clsid in acropdf and log.ThugVulnModules.acropdf_disabled:
        log.warning("Unknown ActiveX Object: %s", clsid)
        raise TypeError()

    # Shockwave Flash
    if clsid in shockwave and log.ThugVulnModules.shockwave_flash_disabled:
        log.warning("Unknown ActiveX Object: %s", clsid)
        raise TypeError()

    # Java Deployment Toolkit
    if clsid in java_deployment_toolkit and log.ThugVulnModules.javaplugin_disabled:
        log.warning("Unknown ActiveX Object: %s", clsid)
        raise TypeError()

    # JavaPlugin
    if clsid.lower().startswith('javaplugin') and log.ThugVulnModules.javaplugin_disabled:
        log.warning("Unknown ActiveX Object: %s", clsid)
        raise TypeError()

    # JavaWebStart
    if clsid.lower().startswith('javawebstart.isinstalled') and log.ThugVulnModules.javaplugin_disabled:
        log.warning("Unknown ActiveX Object: %s", clsid)
        raise TypeError()

    for c in CLSID:
        if clsid in c['id']:
            obj = c
            break

    if obj is None:
        log.warning("Unknown ActiveX object: %s", clsid)
        raise TypeError()

    for method_name, method in obj['methods'].items():
        # _method = new.instancemethod(method, s, s.__class__)
        _method = method.__get__(s, s.__class__)
        setattr(s, method_name, _method)
        methods[method] = _method

    for attr_name, attr_value in obj['attrs'].items():
        setattr(s, attr_name, attr_value)

    # PLEASE REVIEW ME!
    for attr_name, attr_value in obj['funcattrs'].items():
        if 'funcattrs' not in s.__dict__:
            s.__dict__['funcattrs'] = dict()

        s.__dict__['funcattrs'][attr_name] = methods[attr_value]



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLStyleElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    disabled        = False
    media           = attr_property("media")
    type            = attr_property("type")



#!/usr/bin/env python

import sys

class abstractmethod(object):
    def __init__(self, func):
        self.func = func

    def __call__(self, *args, **kwds):
        func_name = self.func.__name__ if sys.version_info.major >= 3 else self.func.func_name
        raise NotImplementedError("method %s is abstract." % func_name)



#!/usr/bin/env python

import six

import logging

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

from .Attr import Attr
from .Node import Node
from .NodeList import NodeList
from .NamedNodeMap import NamedNodeMap

from .Style.CSS.ElementCSSInlineStyle import ElementCSSInlineStyle
log = logging.getLogger("Thug")


FF_STYLES = (
                (27, 'cursor'   ),
                (19, 'font-size'),
            )

FF_INPUTS = (
                (23, 'range'),
            )


class Element(Node, ElementCSSInlineStyle):
    def __init__(self, doc, tag):
        self.tag       = tag
        self.tag._node = self
        Node.__init__(self, doc)
        self.__init_personality()

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        if log.ThugOpts.Personality.browserMajorVersion > 7:
            self.querySelectorAll = self._querySelectorAll
            self.querySelector    = self._querySelector

        if log.ThugOpts.Personality.browserMajorVersion > 8:
            self.getElementsByClassName = self._getElementsByClassName

    def __init_personality_Firefox(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName

    def __init_personality_Chrome(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName

    def __init_personality_Safari(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName

    def __init_personality_Opera(self):
        self.querySelectorAll       = self._querySelectorAll
        self.querySelector          = self._querySelector
        self.getElementsByClassName = self._getElementsByClassName

    def _querySelectorAll(self, selectors):
        try:
            s = self.tag.select(selectors)
        except:  # pylint:disable=bare-except
            return NodeList(self.doc, [])

        return NodeList(self.doc, s)

    def _querySelector(self, selectors):
        from .DOMImplementation import DOMImplementation

        try:
            s = self.tag.select(selectors)
        except:  # pylint:disable=bare-except
            return None

        if s and s[0]:
            return DOMImplementation.createHTMLElement(self, s[0])

        return None

    def __str__(self):
        return str(self.tag)

    def __unicode__(self):
        return unicode(self.tag)

    def __repr__(self):
        return "<Element %s at 0x%08X>" % (self.tag.name, id(self))

    def __eq__(self, other):
        return Node.__eq__(self, other) and hasattr(other, "tag") and self.tag == other.tag

    @property
    def nodeType(self):
        return Node.ELEMENT_NODE

    @property
    def nodeName(self):
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion == 10:
            return self.tagName.upper()

        return self.tagName.lower()

    @property
    def nodeValue(self):
        return None

    @property
    def attributes(self):
        return NamedNodeMap(self)

    @property
    def parentNode(self):
        return Node.wrap(self.doc, self.tag.parent) if self.tag.parent else None

    @property
    def childNodes(self):
        # return Node.wrap(self.doc, NodeList(self.doc, self.tag.contents))
        return NodeList(self.doc, self.tag.contents)

    @property
    def firstChild(self):
        return Node.wrap(self.doc, self.tag.contents[0]) if len(self.tag) > 0 else None

    @property
    def lastChild(self):
        return Node.wrap(self.doc, self.tag.contents[-1]) if len(self.tag) > 0 else None

    @property
    def nextSibling(self):
        return Node.wrap(self.doc, self.tag.next_sibling)

    @property
    def previousSibling(self):
        return Node.wrap(self.doc, self.tag.previous_sibling)

    # Introduced in DOM Level 2
    def hasAttributes(self):
        return self.attributes.length > 0

    # Introduced in DOM Level 2
    def hasAttribute(self, name):
        return self.tag.has_attr(name)

    @property
    def tagName(self):
        return self.tag.name.upper()

    def getAttribute(self, name, flags = 0):
        if not isinstance(name, six.string_types):
            name = str(name)

        if log.ThugOpts.Personality.isIE():
            if log.ThugOpts.Personality.browserMajorVersion < 8:
                # flags parameter is only supported in Internet Explorer earlier
                # than version 8.
                #
                # A value of 0 means that the search is case-insensitive and the
                # returned value does not need to be converted. Other values can
                # be any combination of the following integer constants with the
                # bitwise OR operator:
                #
                # 1   Case-sensitive search
                # 2   Returns the value as a string
                # 4   Returns the value as an URL

                if not flags & 1:
                    name = name.lower()

            value = self.tag[name] if self.tag.has_attr(name) else None

            # FIXME (flags 2 and 4 not implemented yet)
            return value

        return self.tag[name] if self.tag.has_attr(name) else ""

    def setAttribute(self, name, value):
        if not isinstance(name, six.string_types):
            name = str(name)

        if log.ThugOpts.Personality.isFirefox():
            if name in ('style', ):
                svalue = value.split('-')

                _value = svalue[0]
                if len(svalue) > 1:
                    _value = '{}{}'.format(_value, ''.join([s.capitalize() for s in svalue[1:]]))

                for css in [p for p in FF_STYLES if log.ThugOpts.Personality.browserMajorVersion >= p[0]]:
                    if css[1] in value:
                        self.tag[name] = _value
                return

            if name in ('type', ):
                for _input in [p for p in FF_INPUTS if log.ThugOpts.Personality.browserMajorVersion > p[0]]:
                    if _input[1] in value:
                        self.tag[name] = value
                return

        self.tag[name] = value

        if name.lower() in ('src', 'archive'):
            s = urlparse.urlsplit(value)

            handler = getattr(log.SchemeHandler, 'handle_%s' % (s.scheme, ), None)
            if handler:
                handler(self.doc.window, value)
                return

            try:
                response = self.doc.window._navigator.fetch(value, redirect_type = "element workaround")
            except:  # pylint:disable=bare-except
                return

            if response is None:
                return

            if response.status_code == 404:
                return

            ctype = response.headers.get('content-type', None)
            if ctype is None:
                return

            handler = log.MIMEHandler.get_handler(ctype)
            if handler:
                handler(self.doc.window.url, response.content)

    def removeAttribute(self, name):
        del self.tag[name]

    def getAttributeNode(self, name):
        return Attr(self.doc, self, name) if self.tag.has_attr(name) else None

    def setAttributeNode(self, attr):
        self.tag[attr.name] = attr.value

    def removeAttributeNode(self, attr):
        del self.tag[attr.name]

    def getElementsByTagName(self, tagname):
        return NodeList(self.doc, self.tag.find_all(tagname))
        # return self.doc.getElementsByTagName(tagname)

    def _getElementsByClassName(self, classname):
        return NodeList(self.doc, self.tag.find_all(class_ = classname))



# Move Networks Quantum Streaming Player Control
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def UploadLogs(self, url, arg):
    if len(url) > 20000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Move Networks Quantum Streaming Player Control ActiveX",
                                          "Overflow in UploadLogs method")




import logging
log = logging.getLogger("Thug")


def DownloadFile(self, *arg):
    log.ThugLogging.add_behavior_warn('[ZenturiProgramChecker ActiveX] Attack in DownloadFile function')

    log.ThugLogging.add_behavior_warn('[ZenturiProgramChecker ActiveX] Downloading from %s' % (arg[0], ))
    log.ThugLogging.add_behavior_warn("[ZenturiProgramChecker ActiveX] Saving downloaded file as: %s" % (arg[1], ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "ZenturiProgramChecker ActiveX",
                                      "DownloadFile function",
                                      forward = False,
                                      data = {
                                                "url"     : arg[0],
                                                "filename": arg[1]
                                             }
                                     )

    try:
        self._window._navigator.fetch(arg[0], redirect_type = "ZenturiProgramChecker Exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[ZenturiProgramChecker ActiveX] Fetch failed')


def DebugMsgLog(self, *arg):
    log.ThugLogging.add_behavior_warn('[ZenturiProgramChecker ActiveX] Attack in DebugMsgLog function')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "ZenturiProgramChecker ActiveX",
                                      "Attack in DebugMsgLog function")


def NavigateUrl(self, *arg):
    log.ThugLogging.add_behavior_warn('[ZenturiProgramChecker ActiveX] Attack in NavigateUrl function')
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "ZenturiProgramChecker ActiveX",
                                      "Attack in NavigateUrl function")



# Toshiba Surveillance (Surveillix) RecordSend Class (MeIpCamX.DLL 1.0.0.4)
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetPort(self, arg):
    if len(arg) > 10:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Toshiba Surveillance RecordSend Class ActiveX",
                                          "Overflow in SetPort")


def SetIpAddress(self, arg):
    if len(arg) > 18:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Toshiba Surveillance RecordSend Class ActiveX",
                                          "Overflow in SetIpAddress")



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLSelectElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    @property
    def type(self):
        raise NotImplementedError()

    selectedIndex = 0
    value         = None

    @property
    def length(self):
        raise NotImplementedError()

    @property
    def form(self):
        raise NotImplementedError()

    @property
    def options(self):
        raise NotImplementedError()

    disabled        = attr_property("disabled", bool)
    multiple        = attr_property("multiple", bool)
    name            = attr_property("name")
    size            = attr_property("size", thug_long)
    tabIndex        = attr_property("tabindex", thug_long)

    def add(self, element, before):
        raise NotImplementedError()

    def remove(self, index):
        raise NotImplementedError()

    def blur(self):
        raise NotImplementedError()

    def focus(self):
        raise NotImplementedError()



#!/usr/bin/env python
#
# DFT.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import pylibemu
import string
import base64
import random
import logging
import PyV8
import bs4 as BeautifulSoup
import six
from cssutils.parse import CSSParser

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

from .W3C import w3c
from .W3C.DOMImplementation import DOMImplementation
from .W3C.Events.Event import Event
from .W3C.Events.MouseEvent import MouseEvent
from .W3C.Events.HTMLEvent import HTMLEvent
from thug.ActiveX.ActiveX import _ActiveXObject

log = logging.getLogger("Thug")


class DFT(object):
    javascript     = ('javascript', )
    vbscript       = ('vbs', 'vbscript', 'visualbasic')

    # Some event types are directed at the browser as a whole, rather than at
    # any particular document element. In JavaScript, handlers for these events
    # are registered on the Window object. In HTML, we place them on the <body>
    # tag, but the browser registers them on the Window. The following is the
    # complete list of such event handlers as defined by the draft HTML5
    # specification:
    #
    # onafterprint      onfocus         ononline        onresize
    # onbeforeprint     onhashchange    onpagehide      onstorage
    # onbeforeunload    onload          onpageshow      onundo
    # onblur            onmessage       onpopstate      onunload
    # onerror           onoffline       onredo
    window_events = ('abort',
                     'afterprint',
                     'beforeprint',
                     'beforeunload',
                     'blur',
                     'error',
                     'focus',
                     'hashchange',
                     'load',
                     'message',
                     'offline',
                     'online',
                     'pagehide',
                     'pageshow',
                     'popstate',
                     'redo',
                     'resize',
                     'storage',
                     'undo',
                     'unload')

    window_on_events = ['on' + e for e in window_events]

    window_storage_events = ('storage', )
    window_on_storage_events = ['on' + e for e in window_storage_events]
    _on_events = window_on_events + window_on_storage_events

    user_detection_events = ('mousemove', 'scroll', )
    on_user_detection_events = ['on' + e for e in user_detection_events]


    def __init__(self, window, **kwds):
        self.window            = window
        self.window.doc.DFT    = self
        self.anchors           = list()
        self.forms             = kwds['forms'] if 'forms' in kwds else list()
        self.meta              = dict()
        self._context          = None
        log.DFT                = self
        self._init_events()

        PyV8.JSEngine.setStackLimit(10 * 1024 * 1024)

    def _init_events(self):
        self.listeners = list()

        # Events are handled in the same order they are inserted in this list
        self.handled_events = ['load', 'mousemove']

        for event in log.ThugOpts.events:
            self.handled_events.append(event)

        self.handled_on_events = ['on' + e for e in self.handled_events]
        self.dispatched_events = set()

    def __enter__(self):
        return self

    def __exit__(self, _type, value, traceback):
        pass

    @property
    def context(self):
        if self._context is None:
            self._context = self.window.context

        return self._context

    def build_shellcode(self, s):
        i  = 0
        sc = list()

        while i < len(s):
            if s[i] == '"':
                i += 1
                continue

            if s[i] == '%':
                if (i + 6) <= len(s) and s[i + 1] == 'u':
                    currchar = int(s[i + 2: i + 4], 16)
                    nextchar = int(s[i + 4: i + 6], 16)
                    sc.append(chr(nextchar))
                    sc.append(chr(currchar))
                    i += 6
                elif (i + 3) <= len(s) and s[i + 1] == 'u':
                    currchar = int(s[i + 2: i + 4], 16)
                    sc.append(chr(currchar))
                    i += 3
                else:
                    sc.append(s[i])
                    i += 1
            else:
                sc.append(s[i])
                i += 1

        return ''.join(sc)

    def check_URLDownloadToFile(self, emu):
        profile = emu.emu_profile_output

        while True:
            offset = profile.find('URLDownloadToFile')
            if offset < 0:
                break

            profile = profile[offset:]

            p = profile.split(';')
            if len(p) < 2:
                profile = profile[1:]
                continue

            p = p[1].split('"')
            if len(p) < 3:
                profile = profile[1:]
                continue

            url = p[1]
            if url in log.ThugLogging.shellcode_urls:
                return

            try:
                if self.window._navigator.fetch(url, redirect_type = "URLDownloadToFile") is None:
                    log.ThugLogging.add_behavior_warn('[URLDownloadToFile] Fetch failed')

                log.ThugLogging.shellcode_urls.add(url)
            except:  # pylint:disable=bare-except
                log.ThugLogging.add_behavior_warn('[URLDownloadToFile] Fetch failed')

            profile = profile[1:]

    def check_WinExec(self, emu):
        profile = emu.emu_profile_output

        while True:
            offset = profile.find('WinExec')
            if offset < 0:
                break

            profile = profile[offset:]

            p = profile.split(';')
            if not p:
                profile = profile[1:]
                continue

            s = p[0].split('"')
            if len(s) < 2:
                profile = profile[1:]
                continue

            url = s[1]
            if not url.startswith("http"):
                profile = profile[1:]
                continue

            if url in log.ThugLogging.shellcode_urls:
                return

            try:
                if self.window._navigator.fetch(url, redirect_type = "WinExec") is None:
                    log.ThugLogging.add_behavior_warn('[WinExec] Fetch failed')

                log.ThugLogging.shellcode_urls.add(url)
            except:  # pylint:disable=bare-except
                log.ThugLogging.add_behavior_warn('[WinExec] Fetch failed')

            profile = profile[1:]

    def check_shellcode(self, shellcode):
        try:
            sc = self.build_shellcode(shellcode)
        except:  # pylint:disable=bare-except
            sc = shellcode

        emu = pylibemu.Emulator(enable_hooks = False)
        emu.run(sc)

        if emu.emu_profile_output:
            log.ThugLogging.add_shellcode_snippet(emu.emu_profile_output, 'Assembly', 'Shellcode', method = 'Static Analysis')
            log.warning("[Shellcode Profile]\n\n%s", emu.emu_profile_output)
            self.check_URLDownloadToFile(emu)
            self.check_WinExec(emu)

        self.check_url(sc, shellcode)
        emu.free()

    def check_url(self, sc, shellcode):
        schemes = []
        for scheme in ('http://', 'https://'):
            if scheme in sc:
                schemes.append(scheme)

        for scheme in schemes:
            offset = sc.find(scheme)
            if offset == -1:
                continue

            url = sc[offset:]
            url = url.split()[0]
            if url.endswith("'") or url.endswith('"'):
                url = url[:-1]

            if len(url) == 0:
                continue

            i = 0

            while i < len(url):
                if not url[i] in string.printable:
                    break
                i += 1

            url = url[:i]

            log.ThugLogging.add_shellcode_snippet(shellcode, 'Assembly', 'Shellcode', method = 'Static Analysis')
            log.ThugLogging.add_behavior_warn(description = '[Shellcode Analysis] URL Detected: %s' % (url, ), method = 'Static Analysis')

            if url in log.ThugLogging.shellcode_urls:
                return

            try:
                self.window._navigator.fetch(url, redirect_type = "URL found")
                log.ThugLogging.shellcode_urls.add(url)
            except:  # pylint:disable=bare-except
                pass

    def check_shellcodes(self):
        while True:
            try:
                shellcode = log.ThugLogging.shellcodes.pop()
                self.check_shellcode(shellcode)
            except KeyError:
                break

    def check_attrs(self, p):
        for value in p.attrs.values():
            self.check_shellcode(value)

    def shift(self, script, s):
        if script.lower().startswith(s):
            return script[len(s):].lstrip()
        return script

    def fix(self, script):
        script = self.shift(script, 'javascript:')
        script = self.shift(script, 'return')
        return script

    def get_evtObject(self, elem, evtType):
        evtObject = None

        if evtType in MouseEvent.MouseEventTypes:
            evtObject = MouseEvent(evtType, elem)

        if evtType in HTMLEvent.HTMLEventTypes:
            evtObject = HTMLEvent(evtType, elem)

        if evtObject is None:
            return None

        evtObject.eventPhase = Event.AT_TARGET
        evtObject.currentTarget = elem
        return evtObject

    # Events handling
    def handle_element_event(self, evt):
        for (elem, eventType, listener, capture) in self.listeners:  # pylint:disable=unused-variable
            if getattr(elem, 'name', None) is None:
                continue

            if elem.name in ('body', ):
                continue

            if eventType in (evt, ):
                if (elem._node, evt) in self.dispatched_events:
                    continue

                elem._node.dispatchEvent(evt)
                self.dispatched_events.add((elem._node, evt))

    def handle_window_storage_event(self, onevt, evtObject):
        if onevt in self.handled_on_events:
            handler = getattr(self.window, onevt, None)
            if handler:
                handler(evtObject)

    def run_event_handler(self, handler, evtObject):
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
            self.window.event = evtObject
            handler()
        else:
            handler(evtObject)

    def handle_window_event(self, onevt):
        if onevt in self.handled_on_events and onevt not in self.window_on_storage_events:
            count = random.randint(30, 50) if onevt in self.on_user_detection_events else 1

            while count > 0:
                evtObject = self.get_evtObject(self.window, onevt[2:])

                handler = getattr(self.window, onevt, None)
                if handler:
                    self.run_event_handler(handler, evtObject)

                count -= 1

    def handle_document_event(self, onevt):
        if onevt in self.handled_on_events:
            count = random.randint(30, 50) if onevt in self.on_user_detection_events else 1

            while count > 0:
                evtObject = self.get_evtObject(self.window.doc, onevt[2:])

                handler = getattr(self.window.doc, onevt, None)
                if handler:
                    self.run_event_handler(handler, evtObject)

                count -= 1

        # if not getattr(self.window.doc.tag, '_listeners', None):
        #    return

        if '_listeners' not in self.window.doc.tag.__dict__:
            return

        for (eventType, listener, capture) in self.window.doc.tag._listeners:  # pylint:disable=unused-variable
            if eventType not in (onevt[2:], ):
                continue

            count = random.randint(30, 50) if onevt in self.on_user_detection_events else 1

            while count > 0:
                evtObject = self.get_evtObject(self.window.doc, eventType)
                self.run_event_handler(listener, evtObject)
                count -= 1

    def build_event_handler(self, ctx, h):
        # When an event handler is registered by setting an HTML attribute
        # the browser converts the string of JavaScript code into a function.
        # Browsers other than IE construct a function with a single argument
        # named `event'. IE constructs a function that expects no argument.
        # If the identifier `event' is used in such a function, it refers to
        # `window.event'. In either case, HTML event handlers can refer to
        # the event object as `event'.
        if log.ThugOpts.Personality.isIE():
            return ctx.eval("(function() { with(document) { with(this.form || {}) { with(this) { event = window.event; %s } } } }) " % (h, ))

        return ctx.eval("(function(event) { with(document) { with(this.form || {}) { with(this) { %s } } } }) " % (h, ))

    def set_event_handler_attributes(self, elem):
        try:
            attrs = elem.attrs
        except:  # pylint:disable=bare-except
            return

        if 'language' in list(attrs.keys()) and not attrs['language'].lower() in ('javascript', ):
            return

        for evt, h in attrs.items():
            if evt not in self.handled_on_events:
                continue

            self.attach_event(elem, evt, h)

    def attach_event(self, elem, evt, h):
        handler = None

        if isinstance(h, six.string_types):
            handler = self.build_event_handler(self.context, h)
            # PyV8.JSEngine.collect()
        elif isinstance(h, PyV8.JSFunction):
            handler = h
        else:
            try:
                handler = getattr(self.context.locals, h, None)
            except:  # pylint:disable=bare-except
                pass

        if not handler:
            return

        if getattr(elem, 'name', None) and elem.name in ('body', ) and evt in self.window_on_events:
            setattr(self.window, evt, handler)
            return

        if not getattr(elem, '_node', None):
            DOMImplementation.createHTMLElement(self.window.doc, elem)

        elem._node._attachEvent(evt, handler, True)

    def set_event_listeners(self, elem):
        p = getattr(elem, '_node', None)

        if p:
            for evt in self.handled_on_events:
                h = getattr(p, evt, None)
                if h:
                    self.attach_event(elem, evt, h)

        listeners = getattr(elem, '_listeners', None)
        if listeners:
            for (eventType, listener, capture) in listeners:
                if eventType in self.handled_events:
                    self.listeners.append((elem, eventType, listener, capture))

    @property
    def javaUserAgent(self):
        javaplugin = log.ThugVulnModules._javaplugin.split('.')
        last = javaplugin.pop()
        version =  '%s_%s' % ('.'.join(javaplugin), last)
        return log.ThugOpts.Personality.javaUserAgent % (version, )

    @property
    def javaWebStartUserAgent(self):
        javaplugin = log.ThugVulnModules._javaplugin.split('.')
        last = javaplugin.pop()
        version =  '%s_%s' % ('.'.join(javaplugin), last)
        return "JNLP/6.0 javaws/%s (b04) Java/%s" % (version, version, )

    @property
    def shockwaveFlash(self):
        return ','.join(log.ThugVulnModules.shockwave_flash.split('.'))

    def _check_jnlp_param(self, param):
        name  = param.attrs['name']
        value = param.attrs['value']

        if name in ('__applet_ssv_validated', ) and value.lower() in ('true', ):
            log.ThugLogging.log_exploit_event(self.window.url,
                                              'Java WebStart',
                                              'Java Security Warning Bypass (CVE-2013-2423)',
                                              cve = 'CVE-2013-2423')

            log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2013-2423", None)

    def _handle_jnlp(self, data, headers, params):
        try:
            soup = BeautifulSoup.BeautifulSoup(data, "lxml")
        except:  # pylint:disable=bare-except
            return

        jnlp = soup.find("jnlp")
        if jnlp is None:
            return

        codebase = jnlp.attrs['codebase'] if 'codebase' in jnlp.attrs else ''

        log.ThugLogging.add_behavior_warn(description = '[JNLP Detected]', method = 'Dynamic Analysis')

        for param in soup.find_all('param'):
            log.ThugLogging.add_behavior_warn(description = '[JNLP] %s' % (param, ), method = 'Dynamic Analysis')
            self._check_jnlp_param(param)

        jars = soup.find_all("jar")
        if not jars:
            return

        headers['User-Agent'] = self.javaWebStartUserAgent

        for jar in jars:
            try:
                url = "%s%s" % (codebase, jar.attrs['href'], )
                self.window._navigator.fetch(url, headers = headers, redirect_type = "JNLP", params = params)
            except:  # pylint:disable=bare-except
                pass

    def do_handle_params(self, _object):
        params = dict()

        for child in _object.find_all():
            name = getattr(child, 'name', None)
            if name is None:
                continue

            if name.lower() in ('param', ):
                if all(p in child.attrs for p in ('name', 'value', )):
                    params[child.attrs['name'].lower()] = child.attrs['value']

            if name.lower() in ('embed', ):
                self.handle_embed(child)

        if not params:
            return params

        headers = dict()
        headers['Connection'] = 'keep-alive'

        if 'type' in params:
            headers['Content-Type'] = params['type']
        else:
            name = getattr(_object, 'name', None)

            if name in ('applet', ) or 'archive' in params:
                headers['Content-Type'] = 'application/x-java-archive'

            if 'movie' in params:
                headers['x-flash-version'] = self.shockwaveFlash

        if 'Content-Type' in headers and 'java' in headers['Content-Type'] and log.ThugOpts.Personality.javaUserAgent:
            headers['User-Agent'] = self.javaUserAgent

        for key in ('filename', 'movie', ):
            if key not in params:
                continue

            try:
                self.window._navigator.fetch(params[key],
                                             headers = headers,
                                             redirect_type = "params",
                                             params = params)
            except:  # pylint:disable=bare-except
                pass

        for key, value in params.items():
            if key in ('filename', 'movie', 'archive', 'code', 'codebase', 'source', ):
                continue

            if key.lower() not in ('jnlp_href', ) and not value.startswith('http'):
                continue

            try:
                response = self.window._navigator.fetch(value,
                                                        headers = headers,
                                                        redirect_type = "params",
                                                        params = params)

                if response:
                    self._handle_jnlp(response.content, headers, params)
            except:  # pylint:disable=bare-except
                pass

        if 'source' in params:
            try:
                self.window._navigator.fetch(params['source'],
                                             headers = headers,
                                             redirect_type = "params",
                                             params = params)
            except:  # pylint:disable=bare-except
                pass

        if 'archive' not in params and 'code' not in params:
            return params

        if 'codebase' in params:
            archive = urlparse.urljoin(params['codebase'], params['archive'])
        else:
            archive = params['archive']

        try:
            self.window._navigator.fetch(archive,
                                         headers = headers,
                                         redirect_type = "params",
                                         params = params)
        except:  # pylint:disable=bare-except
            pass

        return params

    def handle_object(self, _object):
        log.warning(_object)

        # self.check_attrs(_object)
        params = self.do_handle_params(_object)

        classid  = _object.get('classid', None)
        _id      = _object.get('id', None)
        codebase = _object.get('codebase', None)
        data     = _object.get('data', None)

        if codebase:
            try:
                self.window._navigator.fetch(codebase,
                                             redirect_type = "object codebase",
                                             params = params)
            except:  # pylint:disable=bare-except
                pass

        if data and not data.startswith('data:'):
            try:
                self.window._navigator.fetch(data,
                                             redirect_type = "object data",
                                             params = params)
            except:  # pylint:disable=bare-except
                pass

        if not log.ThugOpts.Personality.isIE():
            return

        # if classid and _id:
        if classid:
            try:
                axo = _ActiveXObject(self.window, classid, 'id')
            except TypeError:
                return

            if _id is None:
                return

            setattr(self.window, _id, axo)
            setattr(self.window.doc, _id, axo)

    def _get_script_for_event_params(self, attr_event):
        params = attr_event.split('(')
        if len(params) < 2:
            return None

        params = params[1].split(')')[0]
        return params.split(',')

    def _handle_script_for_event(self, script):
        attr_for   = script.get("for", None)
        attr_event = script.get("event", None)

        if not attr_for or not attr_event:
            return

        params = self._get_script_for_event_params(attr_event)
        if not params:
            return

        if 'playstatechange' in attr_event.lower():
            with self.context as ctx:
                newState = params.pop()
                ctx.eval("%s = 0;" % (newState.strip(), ))
                try:
                    oldState = params.pop()
                    ctx.eval("%s = 3;" % (oldState.strip(), ))
                except:  # pylint:disable=bare-except
                    pass

    def handle_script(self, script):
        language = script.get('language', None)
        if language is None:
            language = script.get('type', None)

        if language is None:
            _language = 'javascript'
        else:
            _language = language.lower().split('/')[-1]

        handler = getattr(self, "handle_{}".format(_language), None)

        if not handler:
            log.warning("[SCRIPT] Unhandled script type: %s", language)
            return

        if log.ThugOpts.Personality.isIE():
            self._handle_script_for_event(script)

        handler(script)

    def handle_external_javascript_text(self, s, response):
        js = response.content

        # First attempt
        # No specified encoding. This attempt succeeds most of the times.
        try:
            s.text = js
            return True
        except:  # pylint:disable=bare-except
            pass

        # Second attempt
        # Requests will automatically decode content from the server. Most
        # unicode charsets are seamlessly decoded. When you make a request,
        # Requests makes educated guesses about the encoding of the response
        # based on the HTTP headers.
        if response.encoding:
            s.text = response.text
            return True

        # Third (and last) attempt
        # The encoding is (hopefully) detected through the Encoding class.
        enc = log.Encoding.detect(js)
        if enc['encoding'] is None:
            return False

        s.text = js.decode(enc['encoding'])
        return True

    def handle_external_javascript(self, script):
        src = script.get('src', None)
        if src is None:
            return

        try:
            response = self.window._navigator.fetch(src, redirect_type = "script src")
        except:  # pylint:disable=bare-except
            return

        if response is None:
            return

        if response.status_code == 404:
            return

        if not len(response.content):
            return

        if log.ThugOpts.code_logging:
            log.ThugLogging.add_code_snippet(response.content, 'Javascript', 'External')

        s = self.window.doc.createElement('script')

        for attr in script.attrs:
            if attr.lower() not in ('src', ):
                s.setAttribute(attr, script.get(attr))

        if not self.handle_external_javascript_text(s, response):
            return

        try:
            body = self.window.doc.body
        except:  # pylint:disable=bare-except
            body = self.window.doc.getElementsByTagName('body')[0]

        if body:
            body.appendChild(s)

        self.window.evalScript(response.content, tag = script)

    def handle_javascript(self, script):
        log.info(script)

        self.handle_external_javascript(script)

        js = getattr(script, 'text', None)

        if js:
            if log.ThugOpts.code_logging:
                log.ThugLogging.add_code_snippet(js, 'Javascript', 'Contained_Inside')

            self.window.evalScript(js, tag = script)

        self.check_shellcodes()
        self.check_anchors()

    def handle_vbscript(self, script):
        log.info(script)

        if log.ThugOpts.code_logging:
            log.ThugLogging.add_code_snippet(str(script), 'VBScript', 'Contained_Inside')

        log.VBSClassifier.classify(log.ThugLogging.url if log.ThugOpts.local else self.window.url, str(script))
        log.warning("VBScript parsing not available")

    def handle_vbs(self, script):
        self.handle_vbscript(script)

    def handle_visualbasic(self, script):
        self.handle_vbscript(script)

    def handle_noscript(self, script):
        pass

    def handle_form(self, form):
        from .Window import Window

        log.info(form)

        action = form.get('action', None)
        if action is None:
            return

        _action = log.HTTPSession.normalize_url(self.window, action)
        if _action is None:
            return

        if _action in self.forms:
            return

        self.forms.append(_action)
        method = form.get('method', 'get')

        payload = None

        for child in form.find_all():
            name = getattr(child, 'name', None)

            if name.lower() in ('input', ):
                if payload is None:
                    payload = dict()

                if all(p in child.attrs for p in ('name', 'value', )):
                    payload[child.attrs['name']] = child.attrs['value']
        try:
            response = self.window._navigator.fetch(action,
                                                    method = method.upper(),
                                                    body = payload,
                                                    redirect_type = "form")
        except:  # pylint:disable=bare-except
            return

        if response is None:
            return

        if response.status_code == 404:
            return

        ctype = response.headers.get('content-type', None)
        if ctype:
            handler = log.MIMEHandler.get_handler(ctype)
            if handler and handler(action, response.content):
                return

        doc    = w3c.parseString(response.content)
        window = Window(_action, doc, personality = log.ThugOpts.useragent)

        dft = DFT(window, forms = self.forms)
        dft.run()

    def handle_param(self, param):
        log.info(param)

    def handle_embed(self, embed):
        log.warning(embed)

        src = embed.get('src', None)
        if src is None:
            return

        headers = dict()

        embed_type = embed.get('type', None)
        if embed_type:
            headers['Content-Type'] = embed_type

        if 'Content-Type' in headers:
            if 'java' in headers['Content-Type'] and log.ThugOpts.Personality.javaUserAgent:
                headers['User-Agent'] = self.javaUserAgent

            if 'shockwave-flash' in headers['Content-Type']:
                headers['x-flash-version']  = self.shockwaveFlash

        try:
            self.window._navigator.fetch(src, headers = headers, redirect_type = "embed")
        except:  # pylint:disable=bare-except
            pass

    def handle_applet(self, applet):
        log.warning(applet)

        params = self.do_handle_params(applet)

        archive = applet.get('archive', None)
        if not archive:
            return

        headers = dict()
        headers['Connection']   = 'keep-alive'
        headers['Content-type'] = 'application/x-java-archive'

        if log.ThugOpts.Personality.javaUserAgent:
            headers['User-Agent'] = self.javaUserAgent

        try:
            self.window._navigator.fetch(archive,
                                         headers = headers,
                                         redirect_type = "applet",
                                         params = params)
        except:  # pylint:disable=bare-except
            pass

    def handle_meta(self, meta):
        log.info(meta)

        name = meta.get('name', None)
        if name and name.lower() in ('generator', ):
            content = meta.get('content', None)
            if content:
                log.ThugLogging.add_behavior_warn("[Meta] Generator: %s" % (content, ))

        self.handle_meta_http_equiv(meta)

    def handle_meta_http_equiv(self, meta):
        http_equiv = meta.get('http-equiv', None)
        if http_equiv is None:
            return

        content = meta.get('content', None)
        if content is None:
            return

        tag = http_equiv.lower().replace('-', '_')
        handler = getattr(self, 'handle_meta_%s' % (tag, ), None)
        if handler:
            handler(http_equiv, content)

    def handle_meta_x_ua_compatible(self, http_equiv, content):
        # Internet Explorer < 8.0 doesn't support the X-UA-Compatible header
        # and the webpage doesn't specify a <!DOCTYPE> directive.
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion >= 8:
            if http_equiv.lower() in ('x-ua-compatible'):
                self.window.doc.compatible = content

    def force_handle_meta_x_ua_compatible(self):
        for meta in self.window.doc.doc.find_all('meta'):
            http_equiv = meta.get('http-equiv', None)
            if http_equiv is None:
                continue

            if not http_equiv.lower() in ('x-ua-compatible', ):
                continue

            content = meta.get('content', None)
            if content is None:
                continue

            self.handle_meta_x_ua_compatible(http_equiv, content)

    def handle_meta_refresh(self, http_equiv, content):
        from .Window import Window

        if http_equiv.lower() != 'refresh':
            return

        if 'url' not in content.lower():
            return

        url = None
        data_uri = True if 'data:' in content else False

        for s in content.split(';'):
            if data_uri is True and url is not None:
                url = "{};{}".format(url, s)

            s = s.strip()
            if s.lower().startswith('url='):
                url = s[4:]

        if not url:
            return

        if url.startswith("'") and url.endswith("'"):
            url = url[1:-1]

        if url in self.meta and self.meta[url] >= 3:
            return

        if data_uri:
            self._handle_data_uri(url)
            return

        try:
            response = self.window._navigator.fetch(url, redirect_type = "meta")
        except:  # pylint:disable=bare-except
            return

        if response is None:
            return

        if response.status_code == 404:
            return

        if url in self.meta:
            self.meta[url] += 1
        else:
            self.meta[url] = 1

        doc    = w3c.parseString(response.content)
        window = Window(self.window.url, doc, personality = log.ThugOpts.useragent)
        window.open(url)

        dft = DFT(window)
        dft.run()

    def handle_frame(self, frame, redirect_type = 'frame'):
        from .Window import Window

        log.warning(frame)

        src = frame.get('src', None)
        if not src:
            return

        try:
            response = self.window._navigator.fetch(src, redirect_type = redirect_type)
        except:  # pylint:disable=bare-except
            return

        if response is None:
            return

        if response.status_code == 404:
            return

        ctype = response.headers.get('content-type', None)
        if ctype:
            handler = log.MIMEHandler.get_handler(ctype)
            if handler and handler(src, response.content):
                return

        _src = log.HTTPSession.normalize_url(self.window, src)
        if _src:
            src = _src

        doc    = w3c.parseString(response.content)
        window = Window(self.window.url, doc, personality = log.ThugOpts.useragent)
        window.open(src)

        frame_id = frame.get('id', None)
        if frame_id:
            log.ThugLogging.windows[frame_id] = window

        dft = DFT(window)
        dft.run()

    def handle_iframe(self, iframe):
        self.handle_frame(iframe, 'iframe')

    def handle_body(self, body):
        pass

    def do_handle_font_face_rule(self, rule):
        for p in rule.style:
            if p.name.lower() not in ('src', ):
                continue

            url = p.value
            if url.startswith('url(') and len(url) > 4:
                url = url.split('url(')[1].split(')')[0]

            try:
                self.window._navigator.fetch(url, redirect_type = "font face")
            except:  # pylint:disable=bare-except
                return

    def handle_style(self, style):
        log.info(style)

        cssparser = CSSParser(loglevel = logging.CRITICAL, validate = False)

        try:
            sheet = cssparser.parseString(style.text)
        except:  # pylint:disable=bare-except
            return

        for rule in sheet:
            if rule.type == rule.FONT_FACE_RULE:
                self.do_handle_font_face_rule(rule)

    def _handle_data_uri(self, uri):
        """
        Data URI Scheme
        data:[<MIME-type>][;charset=<encoding>][;base64],<data>

        The encoding is indicated by ;base64. If it is present the data is
        encoded as base64. Without it the data (as a sequence of octets) is
        represented using ASCII encoding for octets inside the range of safe
        URL characters and using the standard %xx hex encoding of URLs for
        octets outside that range. If <MIME-type> is omitted, it defaults to
        text/plain;charset=US-ASCII. (As a shorthand, the type can be omitted
        but the charset parameter supplied.)

        Some browsers (Chrome, Opera, Safari, Firefox) accept a non-standard
        ordering if both ;base64 and ;charset are supplied, while Internet
        Explorer requires that the charset's specification must precede the
        base64 token.
        """
        if not uri.lower().startswith("data:"):
            return False

        log.URLClassifier.classify(uri)

        h = uri.split(",")
        if len(h) < 2:
            return False

        data = h[1]
        opts = h[0][len("data:"):].split(";")

        if 'base64' in opts:
            data = base64.b64decode(h[1])
            opts.remove('base64')

        if not opts:
            opts = ["text/plain", "charset=US-ASCII"]

        mimetype = opts[0]

        if mimetype in ('text/html', ):
            from .Window import Window

            doc    = w3c.parseString(data)
            window = Window(self.window.url, doc, personality = log.ThugOpts.useragent)
            window.open(uri)

            dft = DFT(window)
            dft.run()
            return True

        handler = log.MIMEHandler.get_handler(mimetype)
        if handler:
            handler(self.window.url, data)
            return True

        return False

    def handle_a(self, anchor):
        log.info(anchor)

        if log.ThugOpts.extensive:
            log.info(anchor)

            href = anchor.get('href', None)
            if not href:
                return

            if self._handle_data_uri(href):
                return

            try:
                response = self.window._navigator.fetch(href, redirect_type = "anchor")
            except:  # pylint:disable=bare-except
                return

            if response is None:
                return

            if response.status_code == 404:
                return

        self.anchors.append(anchor)

    def handle_link(self, link):
        log.info(link)

        href = link.get('href', None)
        if not href:
            return

        if self._handle_data_uri(href):
            return

        try:
            response = self.window._navigator.fetch(href, redirect_type = "link")
        except:  # pylint:disable=bare-except
            return

        if response is None:
            return

        if response.status_code == 404:
            return

    def check_anchors(self):
        clicked_anchors = [a for a in self.anchors if '_clicked' in a.attrs]
        if not clicked_anchors:
            return

        clicked_anchors.sort(key = lambda anchor: anchor['_clicked'])

        for anchor in clicked_anchors:
            href = anchor['href']
            del anchor['_clicked']

            if 'target' in anchor.attrs and not anchor.attrs['target'] in ('_self', ):
                pid = os.fork()
                if pid == 0:
                    self.follow_href(href)
                else:
                    os.waitpid(pid, 0)
            else:
                self.follow_href(href)

    def follow_href(self, href):
        from .Window import Window

        doc    = w3c.parseString('')
        window = Window(self.window.url, doc, personality = log.ThugOpts.useragent)
        window = window.open(href)

        if window:
            dft = DFT(window)
            dft.run()

    def do_handle(self, child, soup, skip = True):
        name = getattr(child, "name", None)

        if name is None:
            return False

        if skip and name in ('object', 'applet', ):
            return False

        # FIXME: this workaround should be not necessary once the Python 3
        # porting will be completed.
        handler = None

        try:
            handler = getattr(self, "handle_%s" % (str(name.lower()), ), None)
        except:  # pylint:disable=bare-except
            try:
                handler = getattr(self, "handle_%s" % (name.encode('utf-8', 'replace'),  ), None)
            except:  # pylint:disable=bare-except
                pass

        if handler:
            handler(child)
            if name in ('script', ):
                self.run_htmlclassifier(soup)
            return True

        return False

    def run_htmlclassifier(self, soup):
        log.HTMLClassifier.classify(log.ThugLogging.url if log.ThugOpts.local else self.window.url, str(soup))

    def _run(self, soup = None):
        if soup is None:
            soup = self.window.doc.doc

        _soup = soup

        # Dirty hack
        for p in soup.find_all('object'):
            self.handle_object(p)
            self.run_htmlclassifier(soup)

        for p in soup.find_all('applet'):
            self.handle_applet(p)

        for child in soup.descendants:
            self.set_event_handler_attributes(child)
            if not self.do_handle(child, soup):
                continue

            analyzed = set()
            recur    = True

            while recur:
                recur = False

                if tuple(soup.descendants) == tuple(_soup.descendants):
                    break

                for _child in set(soup.descendants) - set(_soup.descendants):
                    if _child not in analyzed:
                        analyzed.add(_child)
                        recur = True

                        name  = getattr(_child, "name", None)
                        if name:
                            self.do_handle(_child, soup, False)

            analyzed.clear()
            _soup = soup

        self.window.doc._readyState = "complete"

        for child in soup.descendants:
            self.set_event_listeners(child)

        for evt in self.handled_on_events:
            try:
                self.handle_window_event(evt)
                self.run_htmlclassifier(soup)
            except:  # pylint:disable=bare-except
                log.warning("[handle_window_event] Event %s not properly handled", evt)

        for evt in self.handled_on_events:
            try:
                self.handle_document_event(evt)
                self.run_htmlclassifier(soup)
            except:  # pylint:disable=bare-except
                log.warning("[handle_document_event] Event %s not properly handled", evt)

        for evt in self.handled_events:
            try:
                self.handle_element_event(evt)
                self.run_htmlclassifier(soup)
            except:  # pylint:disable=bare-except
                log.warning("[handle_element_event] Event %s not properly handled", evt)

    def run(self):
        with self.context as ctx:  # pylint:disable=unused-variable
            self._run()
            self.check_shellcodes()



# Facebook Photo Uploader 4.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def SetExtractIptc(self, val):
    self.__dict__['ExtractIptc'] = val

    if len(val) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "FaceBook Photo Uploader ActiveX",
                                          "Overflow in ExtractIptc property")


def SetExtractExif(self, val):
    self.__dict__['ExtractExif'] = val

    if len(val) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "FaceBook Photo Uploader ActiveX",
                                          "Overflow in ExtractExif property")



# Yahoo! Messenger 8.x YVerInfo.dll ActiveX Control
# CVE-2007-4515

import logging

log = logging.getLogger("Thug")


def fvcom(self, arg0):
    if len(arg0) > 20:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Yahoo! Messenger 8.x YVerInfo.dll ActiveX Control",
                                          "Overflow in fvCom arg0",
                                          cve = 'CVE-2007-4515')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4515", None)

def info(self, arg0):
    if len(arg0) > 20:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Yahoo! Messenger 8.x YVerInfo.dll ActiveX Control",
                                          "Overflow in info arg0",
                                          cve = 'CVE-2007-4515')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-4515", None)




import os
import hashlib
import errno

from thug.Magic.Magic import Magic

import logging

log = logging.getLogger("Thug")

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class TextStream(object):
    def __init__(self):
        self.stream = StringIO()
        self._Line          = 1
        self._Column        = 1
	self._currentLine   = 1
	self._currentColumn = 1 

    @property
    def Line(self):
        return self._Line

    @property
    def Column(self):
        return self._Column

    @property
    def AtEndOfLine(self):
        sstream = self.stream.getvalue().split('\n')
        line    = sstream[self._currentLine]
      
        if len(line[self._currentColumn:]) == 0:
            return True

        return False

    @property
    def AtEndOfStream(self):
        if self._currentLine == self._Line and self._currentColumn == self._Column:
            return True

        return False
		
    def Read(self, characters):
        consume = characters
        sstream = self.stream.getvalue().split('\n')

        result = ""

        while consume > 0:
            line   = sstream[self._currentLine]
            eline  = line[self._currentColumn:]
            length = min(len(eline), consume)

            result  += eline[:length]
            consume -= length

            if consume > 0:
                self._currentLine  += 1
                self._currentColumn = 1
            else:
                self._currentColumn += length

        return result
           
    def ReadLine(self):
        sstream = self.stream.getvalue().split('\n')
        result  = sstream[self._currentLine]
        self._currentLine += 1
        return result

    def ReadAll(self):
        return self.stream.getvalue()

    def Write(self, _string):
	sstring = _string.split('\n')

        if len(sstring) > 1:
	    self._Line  += len(sstring)
	    self._Column = len(sstring[-1]) + 1
        else:
            self._Column += len(_string)

        self.stream.write(_string)

    def WriteLine(self, string):
        self.Write(string + '\n')

    def WriteBlankLines(self, lines):
        self.Write(lines * '\n')

    def Skip(self, characters):
        skip    = characters
        sstream = self.stream.getvalue().split('\n')

        while skip > 0:
            line  = sstream[self._currentLine]
            eline = line[self._currentColumn:]
            
            if skip > len(eline):
                self._currentLine  += 1
                self._currentColumn = 1
                skip -= len(eline)
            else:
                self._currentColumn += skip

    def SkipLine(self):
        self._currentLine += 1

    def Close(self):
        content = self.stream.getvalue()
        log.info(content)

        data = {
            'content' : content,
            'status'  : 200,
            'md5'     : hashlib.md5(content).hexdigest(),
            'sha256'  : hashlib.sha256(content).hexdigest(),
            'fsize'   : len(content),
            'ctype'   : 'textstream',
            'mtype'   : Magic(content).get_mime(),
        }

        log.ThugLogging.log_location(log.ThugLogging.url, data)
        log.TextClassifier.classify(log.ThugLogging.url, content)

        if not log.ThugOpts.file_logging:
            return

	log_dir = os.path.join(log.ThugLogging.baseDir, "analysis", "textstream")

        try:
            os.makedirs(log_dir)
        except OSError as e:
            if e.errno == errno.EEXIST:
                pass
            else:
                raise

        filename = self._filename.split('\\')[-1] if '\\' in self._filename else self._filename
        log_file = os.path.join(log_dir, filename)

        with open(log_file, 'wb') as fd:
            fd.write(content)




from thug.ActiveX.modules import TextStream

import logging
log = logging.getLogger("Thug")


ATTRIBUTES = {
    'Normal'     : 0,      # Normal file. No attributes are set. 
    'ReadOnly'   : 1,      # Read-only file. Attribute is read/write.
    'Hidden'     : 2,      # Hidden file. Attribute is read/write.
    'System'     : 4,      # System file. Attribute is read/write.
    'Volume'     : 8,      # Disk drive volume label. Attribute is read-only.
    'Directory'  : 16,     # Folder or directory. Attribute is read-only.
    'Archive'    : 32,     # File has changed since last backup. Attribute is read/write.
    'Alias'      : 1024,   # Link or shortcut. Attribute is read-only.
    'Compressed' : 2048,   # Compressed file. Attribute is read-only.
}


class File(object):
    def __init__(self, filespec):
        self.Path = filespec
        self._Attributes = ATTRIBUTES['Archive']
        log.ThugLogging.add_behavior_warn('[File ActiveX] Path = %s, Attributes = %s' % (self.Path, self._Attributes, ))

    def getAttributes(self):
        return self._Attributes

    def setAttributes(self, key):
        if key.lower() in ('volume', 'directory', 'alias', 'compressed', ):
            return

        self._attributes = ATTRIBUTES[key]

    Attributes = property(getAttributes, setAttributes)

    @property
    def ShortPath(self):
        _shortPath = []

        for p in self.Path.split('\\'):
            sp = p.split('.')

            if len(sp) == 1:
                spfn = p if len(p) <= 8 else "{}~1".format(p[:6])
            else:
                spfn = ".".join(sp[:-1])
                ext  = sp[-1]

                if len(spfn) > 8:
                    spfn = "{}~1".format(spfn[:6])

                spfn = "{}.{}".format(spfn, ext)

            _shortPath.append(spfn)

        return("\\\\".join(_shortPath))

    @property
    def ShortName(self):
        spath = self.Path.split('\\')
        name  = spath[-1]
        sp    = name.split('.')

        if len(sp) == 1:
            if len(p) <= 8:
                return name
            else:
                return "{}~1".format(name[:6])

        spfn = ".".join(sp[:-1])
        ext = sp[-1]

        if len(spfn) > 8:
            spfn = "{}~1".format(spfn[:6])

        return "{}.{}".format(spfn, ext)

    @property
    def Drive(self):
        spath = self.Path.split('\\')
        if spath[0].endswith(':'):
            return spath[0]

        return 'C:'

    def Copy(self, destination, overwrite = True):
        log.ThugLogging.add_behavior_warn('[File ActiveX] Copy(%s, %s)' % (destination, overwrite, ))

    def Move(self, destination):
        log.ThugLogging.add_behavior_warn('[File ActiveX] Move(%s)' % (destination, ))

    def Delete(self, force = False):
        log.ThugLogging.add_behavior_warn('[File ActiveX] Delete(%s)' % (force, ))

    def OpenAsTextStream(iomode = 'ForReading', _format = 0):
        log.ThugLogging.add_behavior_warn('[File ActiveX] OpenAsTextStream(%s, %s)' % (iomode, _format, ))
        stream = TextStream.TextStream()
        stream._filename = self.Path
        return stream



# Microsoft VFP_OLE_Server

import logging

log = logging.getLogger("Thug")


def foxcommand(self, cmd):
    log.ThugLogging.add_behavior_warn('[Microsoft VFP_OLE_Server ActiveX] Trying to run: %s' % (cmd, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft VFP_OLE_Server ActiveX",
                                      "Trying to run",
                                      data = {
                                                "command": cmd
                                             },
                                      forward = False)



#!/usr/bin/env python
#
# Watchdog.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import signal
import logging

log = logging.getLogger("Thug")


class Watchdog(object):
    def __init__(self, time, callback = None):
        self.time     = time
        self.callback = callback 

    def __enter__(self):
        signal.signal(signal.SIGALRM, self.handler)
        signal.alarm(self.time)

    def __exit__(self, exception_type, exception_value, traceback):
        signal.alarm(0)

    def handler(self, signum, frame):
        log.critical("The analysis took more than %d seconds. Aborting!", self.time)
        if self.callback:
            self.callback(signum, frame)

        log.ThugLogging.log_event()
        os.kill(os.getpid(), signal.SIGTERM)
